<?php

/*
  Convert object to array
 */

function cma_to_array( $obj ) {
    if ( is_object( $obj ) )
        $obj = ( array ) $obj;
    if ( is_array( $obj ) ) {
        $new = array();
        foreach ( $obj as $key => $val ) {
            $new[$key] = cma_to_array( $val );
        }
    } else {
        $new = $obj;
    }

    return $new;
}

/*
 * Search for shortcode by shortcode name
 */

function cma_recursive_array_search( $needle, $haystack ) {
    $val = '';
    foreach ( $haystack as $key => $value ) {
        $current_key = $key;
        if ( is_array( $value ) )
            $val = cma_recursive_array_search( $needle, $value );
        if ( $needle === $value OR ( $val != false and $val != NULL) ) {
            if ( $val == NULL )
                return array( $current_key );
            return array_merge( array( $current_key ), $val );
        }
    }
    return false;
}

/*
 * Search for shortcode by shortcode id
 */

function cma_shortcode_by_id( $layout_data, $id ) {
    $position = array();
    foreach ( $layout_data['rows'] as $rid => $row ) {
        foreach ( $row['grids'] as $gid => $grid ) {
            foreach ( $grid['elements'] as $eid => $element ) {
                if ( $element['id'] == $id ) {
                    array_push( $position, $rid, $gid, $eid );
                }
            }
        }
    }
    return $position;
}

/**
 * Check if we're currently viewing a page that is built with Content Maker.
 *
 * @param bool $can_edit Also check if the user can edit this page
 * @return bool
 */
function cma_page_has_active_cma( $can_edit = false ) {
    global $post;

    if ( is_preview() ) {
        $cma_active = get_post_meta( $post->ID, 'cma_active_preview', true );
        $cma_active = ($cma_active == '1') ? '1' : '0';
    } else {
        // compatibility with previous name/version of plugin
        $cma_active_old = get_post_meta( get_the_ID(), 'content_manager_active', true );

        if ( $cma_active_old == '1' || $cma_active_old == '0' ) {

            $cma_active = (get_post_meta( get_the_ID(), 'cma_layout_data', false ) != '' && get_post_meta( get_the_ID(), 'content_manager_active', true ) == '1' );
            $cma_active = ($cma_active == '1') ? '1' : '0';

            // delete old meta and create new
            delete_post_meta( get_the_ID(), 'content_manager_active' );
            update_post_meta( get_the_ID(), 'cma_active', $cma_active );
        } else {
            $cma_active = (get_post_meta( get_the_ID(), 'cma_layout_data', false ) != '' && get_post_meta( get_the_ID(), 'cma_active', true ) == '1' );
            $cma_active = ($cma_active == '1') ? '1' : '0';
        }
    }

    return $cma_active;
}

/**
 * Get attachment details by attachment id
 * 
 * @param string $attachment_id
 * @return array
 */
function cma_get_attachment( $attachment_id ) {

    $attachment = get_post( $attachment_id );

    if ( !empty( $attachment ) ) {
        return array(
            'alt' => get_post_meta( $attachment->ID, '_wp_attachment_image_alt', true ),
            'caption' => $attachment->post_excerpt,
            'description' => $attachment->post_content,
            'href' => get_permalink( $attachment->ID ),
            'src' => $attachment->guid,
            'title' => $attachment->post_title
        );
    } else {
        return false;
    }
}

/**
 * Get Font Awesome icons list
 * 
 * @param type $path path to font CSS file
 * @return array
 */
function cma_get_font_awesome_classes( $path ) {

    $icons = get_option( 'cma_font_awesome_list' );

    // reset icons list
    //$icons = null;

    if ( empty( $icons ) && !empty( $path ) ) {

        // get icon classes
        $icons = cma_get_icon_font_classes( $path );

        // update options
        if ( !empty( $icons ) ) {
            update_option( 'cma_font_awesome_list', $icons );
        }
    }

    return $icons;
}

/**
 * Get icon font classes by parsing css file
 * 
 * @param string $path
 * @return array
 */
function cma_get_icon_font_classes( $path ) {

    if ( empty( $path ) )
        return;

    if ( file_get_contents( __FILE__ ) && ini_get( 'allow_url_fopen' ) ) {
        $css = file_get_contents( $path );
    } else if ( function_exists( 'curl_version' ) ) {
        $curl = curl_init();
        curl_setopt( $curl, CURLOPT_URL, $path );
        curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );
        $css = curl_exec( $curl );
        curl_close( $curl );
    }


    $pattern = '/\.(fa-(?:\w+(?:-)?)+):before\s+{\s*content:\s*"(.+)";\s+}/';

    preg_match_all( $pattern, $css, $matches, PREG_SET_ORDER );

    $icons = array();
    foreach ( $matches as $match ) {
        $icons[$match[1]] = $match[1];

        // uncomment this for array: class => ASCI
        //$icons[$match[1]] = $match[2];
    }


    return $icons;
}

/*
 *   Get current page id   
 */

function cma_get_current_page_id() {
    // compatibility with Relevanssi plugin for advanced search
    // and with WooCommerce shop plugin
    global $cma_relevanssi_post_id, $post, $cma_yoast_analysis_post_id;

    if ( !empty( $cma_relevanssi_post_id ) ) {
        $post_id = $cma_relevanssi_post_id;
    } else if ( defined( 'PI_WOOCOMMERCE' ) && PI_WOOCOMMERCE && is_shop() ) {
        $post_id = get_option( 'woocommerce_shop_page_id' );
    } else if ( !empty( $cma_yoast_analysis_post_id ) ) {
        $post_id = $cma_yoast_analysis_post_id;
    } else if ( is_404() || empty( $post ) ) {
        $post_id = false;
    } else {
        $post_id = $post->ID;
    }

    return $post_id;
}

/**
 * Sanitize HTML class attribute
 * 
 * @param type $classes
 * @return type array
 */
function cma_sanitize_user_defined_classes( $classes ) {
    $clean_classes = array();

    // fix for version >= v1.1.7
    $html_classes = str_replace( ",", ' ', $classes );

    // split string into array of values
    $html_classes = explode( " ", $html_classes );

    // clean class names
    foreach ( $html_classes as $class ) {
        $clean_classes[] = sanitize_html_class( $class );
    }

    return $clean_classes;
}

add_filter( 'cma_sanitize_user_defined_classes', 'cma_sanitize_user_defined_classes' );
?>