<?php

// register theme custom menu
function volcanno_register_my_custom_menu_page() {
    add_menu_page(THEME_NAME, THEME_NAME, 'manage_options', "volcanno_options", 'volcanno_custom_theme_page', '', 81);
}

add_action('admin_menu', 'volcanno_register_my_custom_menu_page', 9);

// enqueue stylesheets
function volcanno_admin_menu_enqueue_stylesheet($hook) {
    wp_enqueue_style('volcanno_admin_menu', get_template_directory_uri() . '/core/assets/css/admin-menu.css', array(), '1.0', 'screen');
}

add_action('admin_enqueue_scripts', 'volcanno_admin_menu_enqueue_stylesheet');

// template description page
function volcanno_custom_theme_page() {
    global $volcanno_theme_config;

    // get all installed themes
    $themes = wp_get_themes();
    
    // get current theme object
    $current_theme = wp_get_theme();
    ?>
    <div class="volcanno-theme-page">
        <div class="page-header">    
            <h2><?php echo esc_html($current_theme->get('Name')) ?></h2>
            <small><?php echo esc_html($current_theme->get('Description')) ?></small>
        </div>

        <div class="page-content">
            <div class="theme-image">
                <?php
                
                // get theme screenshot
                $screenshot = $current_theme->get_screenshot();

                // in case Child theme is activated and screenshot is not set
                // use parent theme screenshot
                if (is_child_theme() && empty($screenshot)) {
                    
                    // get parent theme name
                    $template = $current_theme->Template;

                    // get theme object
                    $theme = $themes[$template];
                    
                    // get screenshot
                    $screenshot = $theme->get_screenshot();                    
                }

                // output theme screenshot
                echo '<img src="' . esc_url($screenshot) . '" alt="theme screenshot"/>';
                ?>
            </div>
            
            <!-- Theme description -->
            <div class="theme-info">                
                <h2><?php echo esc_html($current_theme->get('Name')) ?></h2>
                <label><?php esc_html_e('Description:', 'volcanno') ?></label>
                <p><?php echo esc_html($current_theme->get('Description')) ?></p>
                <label><?php esc_html_e('Tags:', 'volcanno') ?></label>
                <p><?php echo join(", ", $current_theme->get('Tags')) ?></p>
                <label><?php esc_html_e('Version:', 'volcanno') ?></label>
                <p><?php echo esc_html($current_theme->get('Version')) ?></p>
            </div>

            <!-- Documentation section -->
            <div class="theme-documentation">
                <h2><?php esc_html_e('Documentation', 'volcanno') ?></h2>
                <p><?php esc_html_e('Theme comes with Documentation where you can find how to Setup and use the theme. Documentation can be found in archive downloaded from ThemeForest.', 'volcanno') ?></p>            
            </div>

            <!-- Support section -->
            <div class="theme-support">
                <h2><?php esc_html_e('Support', 'volcanno') ?></h2>
                <p><?php echo wp_kses(__("If you can't find answer to your problem in Documentation please register on our <a href='//support.pixel-industry.com' target='_BLANK'>support forum</a>.
                    We have dedicated team that will help you with your problem.", 'volcanno'), $volcanno_theme_config['allowed_html_tags']) ?><br/>
                    <?php echo wp_kses(__("Note that before registering you will need to obtain Purchase code from ThemeForest. Read <a href='//pixel-industry.com/how-to-obtain-purchase-code/' target='_BLANK'>here</a> how to obtain key.", 'volcanno'), $volcanno_theme_config['allowed_html_tags']) ?>

                </p>
            </div>

            <div class="theme-updates">
                <h2><?php esc_html_e('Updates:', 'volcanno') ?></h2>
                <p><?php esc_html_e('To receive notification for theme updates please install official ThemeForest plugin for updates:', 'volcanno') ?> <a href="<?php echo esc_url(admin_url('admin.php?page=install-required-plugins')); ?>#updates"><?php esc_html_e('Envato Market - Updates.', 'volcanno') ?></a></p>
            </div>
        </div>

    </div>
    <?php
}
