<?php

// Prevent loading file directly
if (!defined('ABSPATH')) {
    exit;
}
if (!class_exists("CMA_Shortcodes_Accordion")) {

    class CMA_Shortcodes_Accordion extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__('Accordion', 'trucking');
            $settings['description'] = esc_html__('Simple UI component.', 'trucking');
            $settings['shortcode'] = 'cma_accordion';
            $settings['nested'] = array('cma_accordion_item');
            $settings['selected_option'] = 'type'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function enqueue_scripts() {
            /* script */
            wp_enqueue_script('cma-scripts-accordion', CMA_CONTENT_URL . 'accordion/js/accordion.js', array('jquery'), '1.0', TRUE);
        }

        function shortcode_html($atts = array(), $content = null) {
            global $volcanno_theme_config;

            $html = '';
            $i = 1;
            $icon_class = '';
            $type_classes = '';

            extract(shortcode_atts(array(
                'type' => 'default',
                'items' => '',
                'active' => ''
                            ), $atts, 'cma_accordion'));

            if ($type == 'large')
                $type_classes = "careers";

            $html .= "<div class='accordion {$type_classes} {$icon_class}'>";

            foreach ($items as $index => $item) {

                $icon_html = '';
                $icon = '';
                $icon_class = '';
                $item_content = do_shortcode($item['content']);
                
                // check if there are HTML in content
                if ($item_content == strip_tags($item_content, '<strong><a>')) {
                    $item_content = wpautop($item_content);
                }

                $item_content = balanceTags($item_content);

                /* Custom icon - not in use
                if ($type == 'default') {
                    if (!empty($item['icon'])) {
                        $icon = $item['icon'];
                    } else {
                        $icon = "fa fa-plus";
                    }
                    $icon_html = "<div class='icon-container'><i class='{$icon}'></i></div>";
                }
                 */

                if ($active == $i) {
                    $active_item = 'active';
                } else {
                    $active_item = '';
                }

                $title = wp_kses($item['title'], $volcanno_theme_config['allowed_html_tags']);

                $html .= "<div class='title {$active_item}'>{$icon_html}<a title='{$title}' href='#tab-{$index}{$i} '>{$title}</a></div>";
                $html .= "<div class='content'>{$item_content}</div>";

                $i++;
            }

            $html .= '</div>';

            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'type' => array(
                    'type' => 'radio',
                    'title' => esc_html__('Type', 'trucking'),
                    'description' => esc_html__('Select accordion type.', 'trucking'),
                    'options' => array(
                        'default' => esc_html__('Default', 'trucking'),
                        'large' => esc_html__('Large title', 'trucking')
                    ),
                    'default' => 'default'
                ),
                'active' => array(
                    'type' => 'number',
                    'title' => esc_html__('Active item', 'trucking'),
                    'description' => esc_html__('Enter index of item that you want to be active e.g 2.', 'trucking'),
                    'default' => '1'
                ),
                'items' => array(
                    'type' => 'group',
                    'title' => esc_html__('Items', 'trucking'),
                    'max' => '',
                    'options' => array(
                        'title' => array(
                            'type' => 'text',
                            'title' => esc_html__('Title', 'trucking'),
                            'description' => esc_html__('Item title.', 'trucking')
                        ),
                        'content' => array(
                            'type' => 'textarea',
                            'title' => esc_html__('Content', 'trucking'),
                            'description' => esc_html__('Add item content. Using HTML in content will automatically disable adding text in paragraph tag.', 'trucking')
                        ),
                        'icon' => array(
                            'type' => 'icon_picker',
                            'title' => esc_html__('Icon', 'trucking'),
                            'description' => esc_html__('Add button icon or leave empty for default plus icon.', 'trucking'),
                            'condition' => array('style', '=', 'icons')
                        ),
                    )
                )
            );
        }

    }

}
?>
