<?php

// Prevent loading file directly
if (!defined('ABSPATH')) {
    exit;
}
if (!class_exists("CMA_Shortcodes_Services")) {

    class CMA_Shortcodes_Services extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__('Services', 'trucking');
            $settings['description'] = esc_html__('Four styles of services.', 'trucking');
            $settings['shortcode'] = 'cma_services';
            $settings['nested'] = false;
            $settings['selected_option'] = 'text'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function enqueue_scripts() {
            /* script */
            wp_enqueue_script('cma-scripts-services', CMA_CONTENT_URL . 'services/js/services.js', array('jquery'), '1.0', TRUE);
        }

        function shortcode_html($atts = array(), $content = null) {
            global $vlc_is_retina, $volcanno_theme_config;
            $animation_class = '';
            $data_animate = '';
            $services_classes = '';
            $params = '';
            $link_target_html = '';
            $read_more_text = '';

            extract(shortcode_atts(array(
                'type' => '',
                'text' => 'Service',
                'description' => '',
                'icon' => 'icon-leaf',
                'icon_white' => '0',
                'animation' => '0',
                'link' => '',
                'custom_link' => '',
                'link_target' => '0',
                'image' => '',
                'readmore' => ''
                            ), $atts, 'cma_services_2'));

            if ($animation == '1') {
                $animation_class = 'animated triggerAnimation';
                $data_animate = 'data-animate="zoomIn"';
            }

            // get service link
            if (!empty($custom_link)) {
                $link = esc_url($custom_link);
            } else if (!empty($link) && $link != '-1') {
                $link = get_permalink($link);
            } else {
                $link = false;
            }

            // services type class
            if ($icon_white == '1') {
                $services_classes = 'white';
            }

            //icon
            if (!empty($icon_linecons)) {
                $icon = $icon_linecons;
            } else {
                $icon = 'fa ' . $icon;
            }

            // subtitle and links
            $subtitle = !empty($subtitle) ? "<p class='subtitle'>{$subtitle}</p>" : '';

            $text = wp_kses($text, $volcanno_theme_config['allowed_html_tags']);
            $description = wp_kses($description, $volcanno_theme_config['allowed_html_tags']);

            // get image or add placeholder
            if (empty($image)) {
                $image_url = TEMPLATEURL . "/img/placeholders/services.jpg";
            } else {
                $image = explode(',', $image);

                //get first image
                $image_url = wp_get_attachment_image_src($image[0], 'full');

                // verify that image isn't .svg
                if (!empty($image_url[0]) && substr($image_url[0], -4) != '.svg') {
                    if ($type == 'service-feature-box') {
                        // crop the image
                        $params = array('width' => 360, 'height' => 216);
                    } else if ($type == 'service-icon-center-boxed') {
                        $params = array('width' => 60, 'height' => 60);
                    } else {
                        $params = array('width' => 100, 'height' => 100);
                    }

                    // filter image size, retina check is hooked on this
                    $params = apply_filters('cma_element_image_dimensions', $params, $this->shortcode_settings['shortcode']);

                    // crop the image
                    $image_url = bfi_thumb($image_url[0], $params);
                } else {
                    $image_url = $image_url[0];
                }
            }
            
            // open link in new tab
            if($link_target == '1')
                $link_target_html = "target='_BLANK'";


            // two services styles
            if ($type == 'service-icon-left') {

                if (!empty($link)) {
                    $title_html = "<a href='{$link}' {$link_target_html}><h3>{$text}</h3></a>";
                } else {
                    $title_html = "<h3>{$text}</h3>";
                }

                if (!empty($image)) {
                    $icon_html = "<img src='{$image_url}' alt='checklist icon'>";
                } else {
                    $icon_html = "<div class='icon {$animation_class}' {$data_animate}>
                                   <i class='{$icon}'></i>
                                </div>";
                }

                $html = "<div class='services-element service-icon-left {$services_classes}'>
                            <div class='icon-container {$animation_class}' {$data_animate}>
                                {$icon_html}
                            </div>

                            <div class='service-details'>
                                {$title_html}

                                <p>{$description}</p>
                            </div>
                        </div>";
            } else if ($type == 'service-icon-center-boxed') {

                if (!empty($link)) {
                    $title_html = "<a href='{$link}' {$link_target_html}><h3>{$text}</h3></a>";
                } else {
                    $title_html = "<h4>{$text}</h4>";
                }

                if (!empty($image)) {
                    $icon_html = "<img src='{$image_url}' alt='checklist icon'>";
                } else {
                    $icon_html = "<div class='icon {$animation_class}' {$data_animate}>
                                   <i class='{$icon}'></i>
                                </div>";
                }

                $html = "<div class='services-element service-icon-center-boxed {$services_classes}'>
                            <div class='service-title'>
                                <div class='icon-container'>
                                    {$icon_html}
                                </div>

                                {$title_html}
                            </div>

                            <p>{$description}</p>
                        </div>";
            } else if ($type == 'service-icon-center') {

                if (!empty($link)) {
                    $title_html = "<a href='{$link}' {$link_target_html}><h3>{$text}</h3></a>";
                } else {
                    $title_html = "<h4>{$text}</h4>";
                }

                if (!empty($image)) {
                    $icon_html = "<img src='{$image_url}' alt='checklist icon'>";
                } else {
                    $icon_html = "<div class='icon {$animation_class}' {$data_animate}>
                                   <i class='{$icon}'></i>
                                </div>";
                }

                $html = "<div class='services-element service-icon-center {$services_classes}'>
                            <div class='icon-container'>
                                {$icon_html}
                            </div>

                            {$title_html}

                            <p>{$description}</p>
                        </div>";
            } else if ($type == 'service-icon-left-boxed') {

                if (!empty($link)) {
                    $title_html = "<a href='{$link}' {$link_target_html}><h3>{$text}</h3></a>";
                } else {
                    $title_html = "<h4>{$text}</h4>";
                }

                if (!empty($image)) {
                    $icon_html = "<div class='icon-container {$animation_class}' {$data_animate}>
                                    <img src='{$image_url}' alt='icon'>
                                    </div>";
                } else {
                    $icon_html = "<div class='icon-container {$animation_class}' {$data_animate}>
                                   <i class='{$icon}'></i>
                                </div>";
                }

                $html = "<div class='services-element service-icon-left-boxed {$services_classes}'>                            
                                {$icon_html}

                                <div class='service-details'>
                                    {$title_html}
                                    <p>{$description}</p>
                                </div>                            
                        </div>";
            } else if ($type == 'service-feature-box') {

                if (!empty($link)) {
                    $title_html = "<a href='{$link}' {$link_target_html}><h3>{$text}</h3></a>";
                } else {
                    $title_html = "<h4>{$text}</h4>";
                }

                if (!empty($image)) {
                    $icon_html = "<img src='{$image_url}' alt='checklist icon'>";
                } else {
                    $icon_html = "<div class='icon {$animation_class}' {$data_animate}>
                                   <i class='{$icon}'></i>
                                </div>";
                }

                if (!empty($readmore) && !empty($link)) {
                    $read_more_text = "<a href='{$link}' class='read-more02' {$link_target_html}><span>
                                {$readmore}
                                <i class='fa fa-chevron-right'></i>
                            </span></a>";

                    $title_html = "<a href='{$link}' {$link_target_html}><h4>{$text}</h4></a>";
                } else if (!empty($link)) {
                    $title_html = "<a href='{$link}' {$link_target_html}><h4>{$text}</h4></a>";
                } else {
                    $title_html = "<h4>{$text}</h4>";
                }

                $html = "<div class='services-element service-feature-box {$services_classes}'>
                            <div class='service-media'>
                                {$icon_html}
                                
                                {$read_more_text}                                
                            </div>

                            <div class='service-body'>
                                <div class='custom-heading'>
                                    {$title_html}
                                </div>

                                <p>{$description}</p>
                            </div>
                        </div>";
            }

            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'type' => array(
                    'type' => 'select',
                    'title' => esc_html__('Type', 'trucking'),
                    'description' => esc_html__('Select services type.', 'trucking'),
                    'options' => array(
                        'service-icon-left' => esc_html__('Icon on left', 'trucking'),
                        'service-icon-center-boxed' => esc_html__('Icon center - boxed', 'trucking'),
                        'service-icon-left-boxed' => esc_html__('Icon left - boxed', 'trucking'),
                        'service-icon-center' => esc_html__('Icon center', 'trucking'),
                        'service-feature-box' => esc_html__('Box', 'trucking'),
                    ),
                    'default' => 'service-icon-left'
                ),
                'text' => array(
                    'type' => 'text',
                    'title' => esc_html__('Title', 'trucking'),
                    'description' => esc_html__('Service title.', 'trucking')
                ),
                'description' => array(
                    'type' => 'textarea',
                    'title' => esc_html__('Description', 'trucking'),
                    'description' => esc_html__('Service description.', 'trucking'),
                    'condition' => array('type', '!=', 'service11')
                ),
                'icon' => array(
                    'type' => 'icon_picker',
                    'title' => esc_html__('Icon', 'trucking'),
                    'description' => esc_html__('Add service icon.', 'trucking'),
                    'condition' => array('type', '!=', 'service-feature-box')
                ),
                'image' => array(
                    'type' => 'image',
                    'title' => esc_html__('Image', 'trucking'),
                    'description' => esc_html__('Upload image for services.', 'trucking')
                ),
                'icon_white' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('White icon', 'trucking'),
                    'description' => esc_html__('Check this to color the icon to white (only for SVG icons).', 'trucking')
                ),
                'animation' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('Animation', 'trucking'),
                    'description' => esc_html__('Enable list items animation.', 'trucking')
                ),
                'link' => array(
                    'type' => 'select',
                    'title' => esc_html__('Link', 'trucking'),
                    'description' => esc_html__('Link to page from this service.', 'trucking'),
                    'data' => 'pages',
                    'default' => ''
                ),
                'link_target' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('New tab', 'trucking'),
                    'description' => esc_html__('Check this to open link in new tab.', 'trucking')
                ),
                'custom_link' => array(
                    'type' => 'text',
                    'title' => esc_html__('Custom URL', 'trucking'),
                    'description' => esc_html__('Enter custom URL for this service.', 'trucking')
                ),
                'readmore' => array(
                    'type' => 'text',
                    'title' => esc_html__('Read More', 'trucking'),
                    'description' => esc_html__('Read More button text.', 'trucking'),
                    'default' => 'Read more',
                    'condition' => array('type', '=', 'service-feature-box')
                ),
            );
        }

    }

}
?>
