<?php

// Prevent loading file directly
if (!defined('ABSPATH')) {
    exit;
}

if (!class_exists("CMA_Shortcodes_Services_1")) {

    class CMA_Shortcodes_Services_1 extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__('Services - Carousel / Gallery', 'trucking');
            $settings['description'] = esc_html__('Services will be rendered as carousel or gallery.', 'trucking');
            $settings['shortcode'] = 'cma_services_1';
            $settings['nested'] = false;
            $settings['selected_option'] = 'type'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function enqueue_scripts() {
            /* script */
            wp_enqueue_script('cma-scripts-services-1', CMA_CONTENT_URL . 'services_1/js/services_1.js', array('jquery', 'owl-carousel'), '1.0', TRUE);
        }

        function shortcode_html($atts = array(), $content = null) {
            global $vlc_is_retina, $volcanno_theme_config;
            $read_more_text = '';
            $services_classes = '';

            extract(shortcode_atts(array(
                'type' => 'carousel',
                'style' => 'horizontal',
                'navigation' => '1',
                'auto_scroll' => '0',
                'timeout' => '3000',
                'services' => '',
                'icon_white' => '0',
                'link_area' => '0',
                'readmore' => ''
                            ), $atts, 'cma_services'));


            // two services styles
            if ($type == 'carousel') {
                $html = "<div class='carousel-container'><div class='services-carousel owl-carousel owl-dot' data-play='{$auto_scroll}' data-timeout='{$timeout}' data-navigation='{$navigation}'>";

                foreach ($services as $index => $service) {
                    $read_more_text = '';
                    $title_html = '';

                    // get image or add placeholder
                    if (empty($service['image'])) {
                        $image_url = TEMPLATEURL . "/img/placeholders/services.jpg";
                    } else {
                        $image = explode(',', $service['image']);

                        //get first image
                        $image_url = wp_get_attachment_image_src($image[0], 'full');

                        // verify that image isn't .svg
                        if (!empty($image_url[0]) && substr($image_url[0], -4) != '.svg') {
                            // crop the image
                            $params = array('width' => 550, 'height' => 330);

                            // filter image size, retina check is hooked on this
                            $params = apply_filters('cma_element_image_dimensions', $params, $this->shortcode_settings['shortcode']);

                            // crop the image
                            $image_url = bfi_thumb($image_url[0], $params);
                        } else {
                            $image_url = $image_url[0];
                        }
                    }

                    // get service link
                    if (!empty($service['custom_link'])) {
                        $link = esc_url($service['custom_link']);
                    } else if (!empty($service['link']) && $service['link'] != '-1') {
                        $link = get_permalink($service['link']);
                    } else {
                        $link = false;
                    }

                    // alt tag
                    $alt = esc_attr($service['text']);

                    $text = wp_kses($service['text'], $volcanno_theme_config['allowed_html_tags']);
                    $description = wp_kses($service['description'], $volcanno_theme_config['allowed_html_tags']);

                    if (!empty($readmore) && !empty($link)) {
                        $readmore = esc_html($readmore);
                        $read_more_text = "<span>
                                {$readmore}
                                <i class='fa fa-chevron-right'></i>
                            </span>";

                        $title_html = "<a href='{$link}'><h4>{$text}</h4></a>";
                    } else if (!empty($link)) {
                        $title_html = "<a href='{$link}'><h4>{$text}</h4></a>";
                    } else {
                        $title_html = "<h4>{$text}</h4>";
                    }

                    $html .= "<div class='owl-item'>
                                <div class='service-feature-box'>
                                    <div class='service-media'>
                                        <img src='{$image_url}' alt='{$alt}'/>

                                        <a href='{$link}' class='read-more02'>
                                            {$read_more_text}
                                        </a>
                                    </div>

                                    <div class='service-body'>
                                        <div class='custom-heading'>
                                            {$title_html}
                                        </div>

                                        <p>{$description}</p>
                                    </div>
                                </div>
                            </div>";
                }

                $html .= "</div></div>";
            } else if ($type == 'gallery') {
                $html = "<ul class='services-gallery clearfix'>";

                foreach ($services as $index => $service) {
                    $read_more_text = '';
                    $title_html = '';


                    // get image or add placeholder
                    if (empty($service['image'])) {
                        $image_url = TEMPLATEURL . "/img/placeholders/services.jpg";
                    } else {
                        $image = explode(',', $service['image']);

                        //get first image
                        $image_url = wp_get_attachment_image_src($image[0], 'full');
                        
                        // verify that image isn't .svg
                        if (substr($image_url[0], -4) != '.svg') {

                            // crop the image
                            $params = array('width' => 550, 'height' => 330);

                            // filter image size, retina check is hooked on this
                            $params = apply_filters('cma_element_image_dimensions', $params, $this->shortcode_settings['shortcode']);

                            // crop the image
                            $image_url = bfi_thumb($image_url[0], $params);
                        } else {
                            $image_url = $image_url[0];
                        }
                    }

                    // alt tag
                    $alt = esc_attr($service['text']);

                    $image_html = "<img src='{$image_url}' alt='{$alt}'>";

                    // get service link
                    if (!empty($service['custom_link'])) {
                        $link = esc_url($service['custom_link']);
                    } else if (!empty($service['link']) && $service['link'] != '-1') {
                        $link = get_permalink($service['link']);
                    } else {
                        $link = false;
                    }

                    $text = wp_kses($service['text'], $volcanno_theme_config['allowed_html_tags']);

                    if (!empty($readmore) && !empty($link)) {
                        $read_more_text = "<div class='hover-details'>
                                             <a href='{$link}'>
                                               <span>
                                               {$readmore}                                
                                               </span>
                                             </a>
                                           </div>";

                        $title_html = "<a href='{$link}'><h3>{$text}</h3></a>";
                    } else {
                        $title_html = "<h3>{$text}</h3>";
                    }

                    $html .= "<li class='col-md-3'>
                            <figure class='service-item-container'>                               
                                <div class='service-item'>
                                    {$image_html}

                                    <div class='hover-mask-container'>
                                        <div class='hover-mask'></div>
                                        {$read_more_text}
                                    </div>
                                </div>
                                <figcaption>
                                    {$title_html}
                                </figcaption>
                            </figure>
                        </li>";
                }

                $html .= "</ul>";
            }

            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'type' => array(
                    'type' => 'radio',
                    'title' => esc_html__('Type', 'trucking'),
                    'description' => esc_html__('Services type.', 'trucking'),
                    'options' => array(
                        'carousel' => esc_html__('Carousel', 'trucking'),
                        'gallery' => esc_html__('Gallery', 'trucking')
                    ),
                    'default' => 'carousel'
                ),
                'auto_scroll' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('Auto Scroll', 'trucking'),
                    'description' => esc_html__('Set services to auto scroll.', 'trucking'),
                    'default' => '0',
                    'condition' => array('type', '=', 'carousel')
                ),
                'navigation' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('Navigation', 'trucking'),
                    'description' => esc_html__('Enable bullets navigation.', 'trucking'),
                    'default' => '1',
                    'condition' => array('type', '=', 'carousel')
                ),
                'timeout' => array(
                    'type' => 'number',
                    'title' => esc_html__('Scroll Timeout', 'trucking'),
                    'description' => esc_html__('Enter scrolling timeout.', 'trucking'),
                    'default' => '3000',
                    'options' => array(
                        'step' => 10
                    ),
                    'condition' => array('type', '=', 'carousel')
                ),
                'readmore' => array(
                    'type' => 'text',
                    'title' => esc_html__('Read More', 'trucking'),
                    'description' => esc_html__('Read More button text.', 'trucking'),
                    'default' => 'Read more'
                ),
                'services' => array(
                    'type' => 'group',
                    'title' => esc_html__('Services', 'trucking'),
                    'max' => '',
                    'options' => array(
                        'text' => array(
                            'type' => 'text',
                            'title' => esc_html__('Title', 'trucking'),
                            'description' => esc_html__('Service title.', 'trucking')
                        ),
                        'description' => array(
                            'type' => 'textarea',
                            'title' => esc_html__('Description', 'trucking'),
                            'description' => esc_html__('Service description.', 'trucking'),
                            'condition' => array(array('type', '=', 'carousel'))
                        ),
                        'image' => array(
                            'type' => 'image',
                            'title' => esc_html__('Image', 'trucking'),
                            'description' => esc_html__('Upload image.', 'trucking')
                        ),
                        'link' => array(
                            'type' => 'select',
                            'title' => esc_html__('Link', 'trucking'),
                            'description' => esc_html__('Link to page from this service.', 'trucking'),
                            'data' => 'pages',
                            'default' => ''
                        ),
                        'custom_link' => array(
                            'type' => 'text',
                            'title' => esc_html__('Custom URL', 'trucking'),
                            'description' => esc_html__('Enter custom URL for this service.', 'trucking')
                        ),
                    )
                )
            );
        }

    }

}
?>
