<?php

// Prevent loading file directly
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

if ( !class_exists( "CMA_Shortcodes_Services_List" ) ) {

    class CMA_Shortcodes_Services_List extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__( 'Services List', 'trucking' );
            $settings['description'] = esc_html__( 'Services will be rendered as list.', 'trucking' );
            $settings['shortcode'] = 'cma_services_list';
            $settings['nested'] = false;
            $settings['selected_option'] = 'type'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function enqueue_scripts() {
            /* script */
            wp_enqueue_script( 'cma-scripts-services-list', CMA_CONTENT_URL . 'services_list/js/services_list.js', array( 'jquery' ), '1.0', TRUE );
        }

        function shortcode_html( $atts = array(), $content = null ) {
            global $vlc_is_retina, $volcanno_theme_config;
            $read_more_text = '';
            $services_classes = '';

            extract( shortcode_atts( array(
                'style' => 'horizontal',
                'navigation' => '1',
                'auto_scroll' => '0',
                'timeout' => '3000',
                'services' => '',
                'icon_white' => '0',
                'link_area' => '0',
                'readmore' => '',
                'new_tab' => '0'
                            ), $atts, 'cma_services' ) );


            // two services styles
            // services type class
            if ( $icon_white == '1' ) {
                $services_classes = 'white';
            }

            if ( $style == 'vertical' ) {
                $html = "<ul class='service-list-big-icons-details clearfix {$services_classes}'>";
            } else if ( $style == 'vertical_simple' ) {
                $html = "<ul class='service-list clearfix {$services_classes}'>";
            } else {
                $html = "<ul class='service-list-big-icons clearfix {$services_classes}'>";
            }

            // services type class
            if ( $icon_white == '1' ) {
                $services_classes = 'white';
            }

            if ( $new_tab == '1' ) {
                $target = "target='_BLANK'";
            } else {
                $target = "";
            }

            foreach ( $services as $index => $service ) {
                $read_more_text = '';
                $title_html = '';


                // get image or add placeholder
                if ( empty( $service['image'] ) ) {
                    $image_url = TEMPLATEURL . "/img/placeholders/services.jpg";
                } else {
                    $image = explode( ',', $service['image'] );

                    //get first image
                    $image_url = wp_get_attachment_image_src( $image[0], 'full' );

                    // verify that image isn't .svg
                    if ( !empty( $image_url[0] ) && substr( $image_url[0], -4 ) != '.svg' ) {

                        // crop the image
                        $params = array( 'width' => 550, 'height' => 330 );

                        // filter image size, retina check is hooked on this
                        $params = apply_filters( 'cma_element_image_dimensions', $params, $this->shortcode_settings['shortcode'] );

                        // crop the image
                        $image_url = bfi_thumb( $image_url[0], $params );
                    } else {
                        $image_url = $image_url[0];
                    }
                }

                // alt tag
                $alt = esc_attr( $service['text'] );

                if ( !empty( $service['image'] ) ) {
                    $icon_html = "<img src='{$image_url}' alt='{$alt}'>";
                } else {
                    $icon_html = "<div class='icon'>
                                   <i class='{$service['icon']}'></i>
                                </div>";
                }

                // get service link
                if ( !empty( $service['custom_link'] ) ) {
                    $link = esc_url( $service['custom_link'] );
                } else if ( !empty( $service['link'] ) && $service['link'] != '-1' ) {
                    $link = get_permalink( $service['link'] );
                } else {
                    $link = false;
                }

                $text = wp_kses( $service['text'], $volcanno_theme_config['allowed_html_tags'] );
                $description = wp_kses( $service['description'], $volcanno_theme_config['allowed_html_tags'] );



                if ( !empty( $readmore ) && !empty( $link ) ) {

                    $title_html = "<a href='{$link}' {$target}><h4>{$text}</h4></a>";
                } else if ( !empty( $link ) ) {
                    $title_html = "<a href='{$link}' {$target}><h4>{$text}</h4></a>";
                } else {
                    $title_html = "<h4>{$text}</h4>";
                }

                if ( $style == 'vertical' ) {
                    $item_text = "<div class='service-details'>
                                    {$title_html}

                                    <p>{$description}</p>
                                </div>";
                } else {
                    $item_text = $title_html;
                }

                $html .= "<li>
                                <div class='icon-container'>
                                    {$icon_html}
                                </div>

                                {$item_text}
                            </li>";
            }

            $html .= "</ul>";

            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'style' => array(
                    'type' => 'radio',
                    'title' => esc_html__( 'Style', 'trucking' ),
                    'description' => esc_html__( 'Style of list items.', 'trucking' ),
                    'options' => array(
                        'horizontal' => esc_html__( 'Horizontal (icon + text)', 'trucking' ),
                        'vertical' => esc_html__( 'Vertical (icon + text + description)', 'trucking' ),
                        'vertical_simple' => esc_html__( 'Vertical (icon + text)', 'trucking' )
                    ),
                    'default' => 'horizontal',
                ),
                'icon_white' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__( 'White icon', 'trucking' ),
                    'description' => esc_html__( 'Check this to color the icon to white (only for SVG icons).', 'trucking' ),
                ),
                'new_tab' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__( 'Target', 'trucking' ),
                    'description' => esc_html__( 'Open link in new tab.', 'trucking' ),
                    'default' => '0'
                ),
                'services' => array(
                    'type' => 'group',
                    'title' => esc_html__( 'Services', 'trucking' ),
                    'max' => '',
                    'options' => array(
                        'text' => array(
                            'type' => 'text',
                            'title' => esc_html__( 'Title', 'trucking' ),
                            'description' => esc_html__( 'Service title.', 'trucking' )
                        ),
                        'description' => array(
                            'type' => 'textarea',
                            'title' => esc_html__( 'Description', 'trucking' ),
                            'description' => esc_html__( 'Service description.', 'trucking' ),
                            'condition' => array( 'style', '==', 'vertical' )
                        ),
                        'icon' => array(
                            'type' => 'icon_picker',
                            'title' => esc_html__( 'Icon', 'trucking' ),
                            'description' => esc_html__( 'Add service icon.', 'trucking' ),
                        ),
                        'image' => array(
                            'type' => 'image',
                            'title' => esc_html__( 'Image', 'trucking' ),
                            'description' => esc_html__( 'Upload image.', 'trucking' )
                        ),
                        'link' => array(
                            'type' => 'select',
                            'title' => esc_html__( 'Link', 'trucking' ),
                            'description' => esc_html__( 'Link to page from this service.', 'trucking' ),
                            'data' => 'pages',
                            'default' => ''
                        ),
                        'custom_link' => array(
                            'type' => 'text',
                            'title' => esc_html__( 'Custom URL', 'trucking' ),
                            'description' => esc_html__( 'Enter custom URL for this service.', 'trucking' )
                        ),
                    )
                )
            );
        }

    }

}
?>
