<?php

// Prevent loading file directly
if (!defined('ABSPATH')) {
    exit;
}
if (!class_exists("CMA_Shortcodes_Slideshow")) {

    class CMA_Shortcodes_Slideshow extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__('Slideshow', 'trucking');
            $settings['description'] = esc_html__('Simple image slideshow.', 'trucking');
            $settings['shortcode'] = 'cma_slideshow';
            $settings['nested'] = false;

            $this->shortcode_settings = $settings;
        }

        function enqueue_scripts() {
            /* script */
            wp_enqueue_script('cma-scripts-slideshow', CMA_CONTENT_URL . 'slideshow/js/slideshow.js', array('jquery', 'nivoslider'), '1.0', TRUE);
        }

        function shortcode_html($atts = array(), $content = null) {
            global $volcanno_theme_config;

            $html_images = '';
            $html_caption = '';

            extract(shortcode_atts(array(
                'images' => '',
                'navigation' => '1',
                'manual_advance' => '1',
                'pause_hover' => '1',
                'pause' => '3000'
                            ), $atts, 'cma_slideshow'));

            // split string of images
            $images = explode(',', $images);

            foreach ($images as $index => $image) {
                //get first image
                $image_url = wp_get_attachment_image_src($image, 'full');

                if (empty($image_url))
                    continue;

                $html_images .= "<img src='{$image_url[0]}' alt='slider image' title='#htmlcaption-{$index}'/>";
                
                // get attachment object
                $attachment = get_post($image);
                
                // use caption if not empty, otherwise post title
                if(!empty($attachment->post_excerpt)){
                    $caption = $attachment->post_excerpt;
                }else{
                    $caption = $attachment->post_title;
                }
                
                // create caption html
                $html_caption .= "<div id='htmlcaption-{$index}' class='nivo-html-caption'>{$caption}</div>";
            }

            $html = "<div class='nivo-wrapper'><div class='nivoSlider' data-manual='{$manual_advance}' data-pause='{$pause}' data-navigation='{$navigation}' data-pause-hover='{$pause_hover}'>{$html_images}</div>{$html_caption}</div>";

            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'images' => array(
                    'type' => 'image',
                    'title' => esc_html__('Images', 'trucking'),
                    'description' => esc_html__('Upload slider images.', 'trucking')
                ),
                'navigation' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('Show navigation', 'trucking'),
                    'description' => esc_html__('Check this to show navigation.', 'trucking'),
                    'default' => '1'
                ),
                'manual_advance' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('Force manual transitions', 'trucking'),
                    'description' => esc_html__('Disable automatic sliding.', 'trucking'),
                    'default' => '1'
                ),
                'pause_hover' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__('Hover pause', 'trucking'),
                    'description' => esc_html__('Pause sliding on hover.', 'trucking'),
                    'default' => '1'
                ),
                'pause' => array(
                    'type' => 'number',
                    'title' => esc_html__('Slide Pause', 'trucking'),
                    'description' => esc_html__('Enter sliding timeout.', 'trucking'),
                    'default' => '3000',
                    'options' => array(
                        'step' => 10
                    )
                )
            );
        }

    }

}
?>
