<?php

/* -------------------------------------------------------------------- 
 * Post formats and portfolio template scripts
  -------------------------------------------------------------------- */

function volcanno_enqueue_additional_metabox_scripts() {
    $screen = get_current_screen();

    if ($screen->base == 'post') {
        wp_enqueue_script('rwmb-custom-scripts', TEMPLATEURL . '/core/meta-box/custom-fields/custom-scripts.js', array('jquery'), '1.0', true);
    }
}

if (is_admin()) {
    add_action('admin_enqueue_scripts', 'volcanno_enqueue_additional_metabox_scripts');
}

/* -------------------------------------------------------------------- 
 * MetaBox for Custom Post Types
  -------------------------------------------------------------------- */

function volcanno_register_meta_boxes($meta_boxes) {
    global $volcanno_theme_config;

    // Get avaliable pages
    $all_avaliable_pages = get_pages('sort_column=post_parent,menu_order');
    $pages[''] = 'Select a page';
    foreach ($all_avaliable_pages as $page) {
        $pages[$page->ID] = $page->post_title;
    }

    // Define a meta box for Pages

    $prefix = 'pg_';

    $meta_boxes[] = array('title' => esc_html__('Page Options', 'trucking'),
        'pages' => array('page'),
        // Works with Meta Box Tabs plugin
        'tabs' => array(
            'page_title' => esc_html__('Page title', 'trucking'),
            'vehicles' => esc_html__('Vehicles options', 'trucking'),
            'call_to_action' => esc_html__('Call to Action', 'trucking'),
        ),
        'fields' => array(
            array(
                'name' => esc_html__('Sidebar', 'trucking'),
                'desc' => esc_html__('Page can have left or right sidebar.', 'trucking'),
                'id' => "{$prefix}sidebar",
                'type' => 'image_select',
                'std' => 'fullwidth',
                'options' => array(
                    'fullwidth' => TEMPLATEURL . '/core/assets/images/fullwidth.png',
                    'left' => TEMPLATEURL . '/core/assets/images/left.png',
                    'right' => TEMPLATEURL . '/core/assets/images/right.png'
                ),
                'tab' => 'page_title',
            ),
            array(
                'name' => esc_html__('Hide page title', 'trucking'),
                'id' => "{$prefix}hide_title",
                'desc' => esc_html__('Check this to hide page title.', 'trucking'),
                'type' => 'checkbox',
                'std' => '0',
                'tab' => 'page_title'
            ),
            array(
                'name' => esc_html__('Nice title', 'trucking'),
                'id' => "{$prefix}page_description",
                'desc' => esc_html__('Nice title will replace page title.', 'trucking'),
                'type' => 'text',
                'std' => '',
                'tab' => 'page_title'
            ),
            array(
                'name' => esc_html__('Background color', 'trucking'),
                'desc' => esc_html__('Set title background color.', 'trucking'),
                'id' => "{$prefix}title_color",
                'type' => 'color',
                'tab' => 'page_title'
            ),
            // Image Upload
            array(
                'name' => esc_html__('Background image', 'trucking'),
                'desc' => esc_html__('Set background image..', 'trucking'),
                'id' => "{$prefix}title_image",
                'type' => 'image_advanced',
                'max_file_uploads' => 1,
                'tab' => 'page_title'
            ),
            /*
             * VEHICLES
             */
            array(
                'name' => esc_html__('Style', 'trucking'),
                'id' => "{$prefix}vehicles_style",
                'type' => 'select',
                'options' => array(
                    'full' => esc_html__('Full width', 'trucking'),
                    'grid' => esc_html__('Grid', 'trucking'),
                ),
                'std' => '1-col',
                'desc' => esc_html__('Select page style.', 'trucking'),
                'tab' => 'vehicles'
            ),
            array(
                'name' => esc_html__('Order', 'trucking'),
                'id' => "{$prefix}vehicles_order",
                'type' => 'select',
                'options' => array(
                    'ASC' => esc_html__('Ascending', 'trucking'),
                    'DESC' => esc_html__('Descending', 'trucking')
                ),
                'std' => 'ASC',
                'desc' => esc_html__('Select items order.', 'trucking'),
                'tab' => 'vehicles'
            ),
            array(
                'name' => esc_html__('Order By', 'trucking'),
                'id' => "{$prefix}vehicles_order_by",
                'type' => 'select',
                'options' => array(
                    'id' => esc_html__('ID', 'trucking'),
                    'title' => esc_html__('Title', 'trucking'),
                    'date' => esc_html__('Date', 'trucking'),
                    'rand' => esc_html__('Random', 'trucking'),
                ),
                'std' => 'id',
                'desc' => esc_html__('Order items by one of available parameters.', 'trucking'),
                'tab' => 'vehicles'
            ),
            array(
                'name' => esc_html__('Pagination', 'trucking'),
                'id' => "{$prefix}vehicles_pagination",
                'desc' => esc_html__('Number of items per page.', 'trucking'),
                'type' => 'slider',
                'std' => '12',
                'tab' => 'vehicles'
            ),
            array(
                'name' => esc_html__('Lightbox', 'trucking'),
                'desc' => esc_html__('Change Lightbox options.', 'trucking'),
                'id' => "{$prefix}vehicles_lightbox_options",
                'type' => 'radio',
                'options' => array(
                    'animated' => 'Animated hover button',
                    'image_link' => 'Image as lightbox link',
                    'disabled' => 'Disabled'
                ),
                'std' => 'animated',
                'tab' => 'vehicles'
            ),
            array(
                'name' => esc_html__('Images from project', 'trucking'),
                'id' => "{$prefix}vehicles_project_images",
                'desc' => esc_html__('In lightbox show images from project instead gallery of all project images.', 'trucking'),
                'type' => 'checkbox',
                'std' => '0',
                'tab' => 'vehicles'
            ),
            array(
                'name' => esc_html__('Include Categories', 'trucking'),
                'id' => $prefix . 'vehicles_include_cats',
                'desc' => esc_html__('Enter categories that you want to show on this page (category IDs)', 'trucking'),
                'type' => 'number',
                'clone' => true,
                'tab' => 'vehicles'
            ),
            /*
             * VEHICLES - CALL TO ACTION
             */
            array(
                'name' => esc_html__('Show Call to Action', 'trucking'),
                'id' => "{$prefix}vehicles_cta",
                'desc' => esc_html__('Show Call to Action section.', 'trucking'),
                'type' => 'checkbox',
                'std' => '0',
                'tab' => 'call_to_action'
            ),
            array(
                'name' => esc_html__('Title', 'trucking'),
                'id' => "{$prefix}vehicles_cta_title",
                'desc' => esc_html__('Call to action title.', 'trucking'),
                'type' => 'text',
                'std' => 'Title',
                'tab' => 'call_to_action',
                'visible' => array('pg_vehicles_cta', true)
            ),
            array(
                'name' => esc_html__('Subtitle', 'trucking'),
                'id' => "{$prefix}vehicles_cta_subtitle",
                'desc' => esc_html__('Call to action subtitle.', 'trucking'),
                'type' => 'textarea',
                'std' => 'Subtitle',
                'tab' => 'call_to_action',
                'visible' => array('pg_vehicles_cta', true)
            ),
            array(
                'name' => esc_html__('Button', 'trucking'),
                'id' => "{$prefix}vehicles_show_cta_button",
                'desc' => esc_html__('Show Call to Action button.', 'trucking'),
                'type' => 'checkbox',
                'std' => '0',
                'tab' => 'call_to_action',
                'visible' => array('pg_vehicles_cta', true)
            ),
            array(
                'name' => esc_html__('Button text', 'trucking'),
                'id' => "{$prefix}vehicles_cta_button_text",
                'desc' => esc_html__('Enter button text.', 'trucking'),
                'type' => 'text',
                'std' => 'Apply now',
                'tab' => 'call_to_action',
                'visible' => array(
                    array('pg_vehicles_cta', true),
                    array('pg_vehicles_show_cta_button', true)
                )
            ),
            array(
                'name' => esc_html__('Button URL', 'trucking'),
                'id' => "{$prefix}vehicles_cta_button_url",
                'desc' => esc_html__('Enter button URL.', 'trucking'),
                'type' => 'text',
                'std' => '',
                'tab' => 'call_to_action',
                'visible' => array(
                    array('pg_vehicles_cta', true),
                    array('pg_vehicles_show_cta_button', true)
                )
            ),
            array(
                'name' => esc_html__('Background color', 'trucking'),
                'desc' => esc_html__('Set section background color.', 'trucking'),
                'id' => "{$prefix}vehicles_cta_bkg_color",
                'type' => 'color',
                'tab' => 'call_to_action',
                'visible' => array('pg_vehicles_cta', true)
            ),
            // Image Upload
            array(
                'name' => esc_html__('Background image', 'trucking'),
                'desc' => esc_html__('Set background image.', 'trucking'),
                'id' => "{$prefix}vehicles_cta_bkg_image",
                'type' => 'image_advanced',
                'max_file_uploads' => 1,
                'tab' => 'call_to_action',
                'visible' => array('pg_vehicles_cta', true)
            ),
        )
    );

    // Define a meta box for Blog

    $prefix = 'pt_';
    $meta_boxes[] = array(
        'title' => esc_html__('Post Format Options', 'trucking'),
        'pages' => array('post'),
        'fields' => array(
            array(
                'type' => 'heading',
                'name' => esc_html__('No options for this post format.', 'trucking'),
                'id' => 'post_format_options', // Not used but needed for plugin
            ),
            // Video URL
            array(
                'name' => esc_html__('Video Embed Code', 'trucking'),
                'id' => $prefix . 'video_url',
                'desc' => esc_html__('Please enter embed code (iframe).', 'trucking'),
                'type' => 'textarea',
                'std' => '',
            ),
            // Quote text
            array(
                'name' => esc_html__('Quote text', 'trucking'),
                'id' => $prefix . 'quote_text',
                'desc' => esc_html__('Please enter here quote text.', 'trucking'),
                'type' => 'textarea',
                'std' => '',
            ),
            array(
                'name' => esc_html__('Author', 'trucking'),
                'id' => $prefix . 'quote_author',
                'desc' => esc_html__('Please enter quote author.', 'trucking'),
                'type' => 'text',
                'std' => '',
            ),
            // Audio files
            array(
                'name' => esc_html__('Audio files', 'trucking'),
                'id' => $prefix . 'audio_files',
                'desc' => wp_kses(__('Please enter link to audio file(s), one per line. Check <a href="//www.jplayer.org/latest/developer-guide/#reference-html5-audio-format">here</a> which audio formats are supported by popular browsers.', 'trucking'), $volcanno_theme_config['allowed_html_tags']),
                'type' => 'textarea',
                'std' => '',
            ),
            // Gallery images
            array(
                'name' => esc_html__('Gallery images', 'trucking'),
                'id' => $prefix . 'gallery_images',
                'desc' => wp_kses(__('Upload images for gallery. <strong>Minimum size:</strong> 620px x 308px', 'trucking'), $volcanno_theme_config['allowed_html_tags']),
                'type' => 'image_advanced'
            ),
            // Link
            array(
                'name' => esc_html__('Link', 'trucking'),
                'id' => $prefix . 'link_url',
                'desc' => esc_html__('Enter URL that will be used for Link post format.', 'trucking'),
                'type' => 'text'
            )
        )
    );

    // Define a meta box for Vehicles

    $prefix = 'vh_';
    $meta_boxes[] = array(
        'title' => esc_html__('Item Customization', 'trucking'),
        'pages' => array('pi_vehicles'),
        'fields' => array(
            array(
                'name' => esc_html__('Page title', 'trucking'),
                'id' => "{$prefix}page_title",
                'desc' => esc_html__('Show Page title section.', 'trucking'),
                'type' => 'checkbox',
                'std' => '1',
                'tab' => 'single',
                'tab' => 'content'
            ),
            array(
                'name' => esc_html__('Format', 'trucking'),
                'desc' => esc_html__('Upload image or video.', 'trucking'),
                'id' => "{$prefix}gallery_format",
                'type' => 'radio',
                'std' => 'image',
                'options' => array(
                    'image' => 'Image',
                    'video' => 'Video',
                ),
            ),
            // Image Upload
            array(
                'name' => esc_html__('Image', 'trucking'),
                'desc' => esc_html__('Use to upload image(s). Minimum image size: 540px x 363px. In case of Video item it will be used as poster.', 'trucking'),
                'id' => "{$prefix}image",
                'type' => 'image_advanced',
                'tab' => 'basic'
            ),
            array(
                'name' => esc_html__('Video URL', 'trucking'),
                'id' => $prefix . 'video_url',
                'desc' => esc_html__('Enter the video URL.', 'trucking'),
                'type' => 'text',
                'std' => '',
                'tab' => 'basic',
                'visible' => array('vh_gallery_format', '=', 'video')
            ),
            array(
                'name' => esc_html__('Style', 'trucking'),
                'desc' => esc_html__('Render images as slider or as grid of images (single page).', 'trucking'),
                'id' => "{$prefix}gallery_style",
                'type' => 'select',
                'std' => 'slider',
                'options' => array(
                    'slider' => 'Slider',
                    'grid' => 'Grid',
                ),
                'visible' => array('vh_gallery_format', '=', 'image')
            ),
            array(
                'name' => esc_html__('Hide Arrows', 'trucking'),
                'id' => "{$prefix}slider_hide_arrows",
                'desc' => esc_html__('Hide slider arrows.', 'trucking'),
                'type' => 'checkbox',
                'std' => '0',
                'tab' => 'slider',
                'visible' => array('vh_gallery_style', '=', 'slider')
            ),
            array(
                'name' => esc_html__('Auto Slide', 'trucking'),
                'id' => "{$prefix}slider_autoslide",
                'desc' => esc_html__('Enable autosliding.', 'trucking'),
                'type' => 'checkbox',
                'std' => '0',
                'tab' => 'slider',
                'visible' => array('vh_gallery_style', '=', 'slider')
            ),
            array(
                'name' => esc_html__('Pause Time', 'trucking'),
                'id' => $prefix . 'slider_pause_time',
                'desc' => esc_html__('Enter time between two slides.', 'trucking'),
                'type' => 'number',
                'std' => '3000',
                'visible' => array('vh_gallery_style', '=', 'slider')
            ),
        )
    );

    return $meta_boxes;
}

add_filter('rwmb_meta_boxes', 'volcanno_register_meta_boxes');
?>
