<?php

/**
 * ReduxFramework Sample Config File
 * For full documentation, please visit: http://docs.reduxframework.com/
 */
if (!class_exists('Redux')) {
    return;
}


// This is your option name where all the Redux data is stored.
$opt_name = "volcanno_options";


$theme = wp_get_theme(); // For use with some settings. Not necessary.

$args = array(
    // TYPICAL -> Change these values as you need/desire
    'opt_name' => 'volcanno_options',
    // This is where your data is stored in the database and also becomes your global variable name.
    'display_name' => $theme->get('Name'),
    // Name that appears at the top of your panel
    'display_version' => $theme->get('Version'),
    // Version that appears at the top of your panel
    'menu_type' => 'submenu',
    //Specify if the admin menu should appear or not. Options: menu or submenu (Under appearance only)
    'allow_sub_menu' => true,
    // Show the sections below the admin menu item or not
    'menu_title' => esc_html__('Theme Options', 'trucking'),
    'page_title' => esc_html__('Theme Options', 'trucking'),
    // You will need to generate a Google API key to use this feature.
    // Please visit: https://developers.google.com/fonts/docs/developer_api#Auth
    'google_api_key' => 'AIzaSyBsN1cG-NVXTUyefbmSlbv5NxMWyDzD8Nw',
    // Set it you want google fonts to update weekly. A google_api_key value is required.
    'google_update_weekly' => false,
    // Must be defined to add google fonts to the typography module
    'async_typography' => true,
    // Use a asynchronous font on the front end or font string
    //'disable_google_fonts_link' => true,                    // Disable this in case you want to create your own google fonts loader
    'admin_bar' => true,
    // Show the panel pages on the admin bar
    'admin_bar_icon' => 'dashicons-portfolio',
    // Choose an icon for the admin bar menu
    'admin_bar_priority' => 50,
    // Choose an priority for the admin bar menu
    'global_variable' => '',
    // Set a different name for your global variable other than the opt_name
    'dev_mode' => false,
    // Show the time the page took to load, etc
    'update_notice' => true,
    // If dev_mode is enabled, will notify developer of updated versions available in the GitHub Repo
    'customizer' => true,
    // Enable basic customizer support
    //'open_expanded'     => true,                    // Allow you to start the panel in an expanded way initially.
    //'disable_save_warn' => true,                    // Disable the save warning when a user changes a field
    // OPTIONAL -> Give you extra features
    'page_priority' => null,
    // Order where the menu appears in the admin area. If there is any conflict, something will not show. Warning.
    'page_parent' => "volcanno_options",
    // For a full list of options, visit: http://codex.wordpress.org/Function_Reference/add_submenu_page#Parameters
    'page_permissions' => 'manage_options',
    // Permissions needed to access the options panel.
    'menu_icon' => '',
    // Specify a custom URL to an icon
    'last_tab' => '',
    // Force your panel to always open to a specific tab (by id)
    'page_icon' => 'icon-themes',
    // Icon displayed in the admin panel next to your menu_title
    'page_slug' => '_vlc_options',
    // Page slug used to denote the panel
    'save_defaults' => true,
    // On load save the defaults to DB before user clicks save or not
    'default_show' => false,
    // If true, shows the default value next to each field that is not the default value.
    'default_mark' => '',
    // What to print by the field's title if the value shown is default. Suggested: *
    'show_import_export' => true,
    // Shows the Import/Export panel when not used as a field.
    // CAREFUL -> These options are for advanced use only
    'transient_time' => 60 * MINUTE_IN_SECONDS,
    'output' => true,
    // Global shut-off for dynamic CSS output by the framework. Will also disable google fonts output
    'output_tag' => true,
    // Allows dynamic CSS to be generated for customizer and google fonts, but stops the dynamic CSS from going to the head
    // 'footer_credit'     => '',                   // Disable the footer credit of Redux. Please leave if you can help it.
    // FUTURE -> Not in use yet, but reserved or partially implemented. Use at your own risk.
    'database' => '',
    // possible: options, theme_mods, theme_mods_expanded, transient. Not fully functional, warning!
    'system_info' => false,
    
);

// SOCIAL ICONS -> Setup custom links in the footer for quick links in your panel footer icons.		
$args['share_icons'][] = array(
    'url' => 'https://github.com/pixel-industry/',
    'title' => 'Visit us on GitHub',
    'icon' => 'el-icon-github'
// 'img' => '', // You can use icon OR img. IMG needs to be a full URL.
);
$args['share_icons'][] = array(
    'url' => 'https://www.facebook.com/pixel.industry.themes',
    'title' => 'Like us on Facebook',
    'icon' => 'el-icon-facebook'
);
$args['share_icons'][] = array(
    'url' => 'https://twitter.com/pixel_industry',
    'title' => 'Follow us on Twitter',
    'icon' => 'el-icon-twitter'
);
$args['share_icons'][] = array(
    'url' => 'http://www.linkedin.com/company/redux-framework',
    'title' => 'Find us on LinkedIn',
    'icon' => 'el-icon-linkedin'
);
$args['share_icons'][] = array(
    'url' => 'http://dribbble.com/pixel_industry',
    'title' => 'Our Work on Dribbble',
    'icon' => 'el-icon-dribbble'
);
$args['share_icons'][] = array(
    'url' => 'https://www.behance.net/pixel-industry',
    'title' => 'Our Portfolio on Behance',
    'icon' => 'el-icon-behance'
);



Redux::setArgs($opt_name, $args);

/*
 *
 * ---> START SECTIONS
 *
 */


// ACTUAL DECLARATION OF SECTIONS


Redux::setSection($opt_name, array(
    'icon' => 'el-icon-tint',
    'title' => esc_html__('Appearance', 'trucking'),
    'fields' => array(
        array(
            'id' => 'predefined_color_style',
            'type' => 'select',
            'title' => esc_html__('Color style', 'trucking'),
            'subtitle' => esc_html__('Choose on of available color styles or set custom in option below.', 'trucking'),
            'options' => array(
                'color-default' => 'Default',
                'color-aqua' => 'Aqua',
                'color-blue-aqua' => 'Aqua Blue',
                'color-blue-green' => 'Blue-Green',
                'color-blue-orange' => 'Blue Orange',
                'color-blue-red' => 'Blue Red',
                'color-blue' => 'Blue',
                'color-green' => 'Green',
                'color-orange' => 'Orange',
                'color-red' => 'Red'
            ),
            'default' => 'color-default',
        ),
        array(
            'id' => 'custom_color_style',
            'type' => 'color',
            'output' => array('.site-title'),
            'title' => esc_html__('Custom Color Style', 'trucking'),
            'subtitle' => esc_html__('Set custom color that will be applied to all elements..', 'trucking'),
            'default' => '',
            'validate' => 'color',
        ),
        array(
            'id' => 'scroll_to_top',
            'type' => 'checkbox',
            'title' => esc_html__('Scroll to Top', 'trucking'),
            'subtitle' => esc_html__('Show Scroll to Top button for easier navigation.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'responsive',
            'type' => 'checkbox',
            'title' => esc_html__('Responsive', 'trucking'),
            'subtitle' => esc_html__('Unheck to disable responsivness.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'retina',
            'type' => 'checkbox',
            'title' => esc_html__('HiDPI images', 'trucking'),
            'subtitle' => esc_html__('Enable HiDPI images feature (Retina).', 'trucking'),
            'default' => '0'// 1 = on | 0 = off
        ),
        array(
            'id' => 'custom_css',
            'type' => 'textarea',
            'title' => esc_html__('Custom CSS', 'trucking'),
            'subtitle' => esc_html__('Quickly add some CSS to your theme by adding it to this block.', 'trucking'),
            'validate' => 'css',
            'output' => 'body'
        )
    )
));

Redux::setSection($opt_name, array(
    'icon' => 'el-icon-cogs',
    'title' => esc_html__('Header', 'trucking'),
    'fields' => array(
        array(
            'id' => 'header_style',
            'type' => 'select_image',
            'title' => esc_html__('Style', 'trucking'),
            'subtitle' => esc_html__('Select header style.', 'trucking'),
            'options' => array(
                'header-style01' => array('alt' => 'Header - default', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-default.jpg'),
                'header-style01-tp' => array('alt' => 'Header - default with top bar', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-default-top-bar.jpg'),
                'header-style02-l' => array('alt' => 'Header - simple with light top bar', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-simple-light-top-bar.jpg'),
                'header-style02-d' => array('alt' => 'Header - simple with dark top bar', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-simple-dark-top-bar.jpg'),
                'header-style02-d-tp' => array('alt' => 'Simple dark', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-simple-dark.jpg'),
                'header-style03-l' => array('alt' => 'Header - menu bottom light', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-menu-bottom-light.jpg'),
                'header-style03-d' => array('alt' => 'Header - menu bottom dark', 'img' => TEMPLATEURL . '/img/theme-options/headers/header-menu-bottom-dark.jpg'),
            ), //Must provide key => value(array:title|img) pairs for radio options
            'default' => 'header-style01'
        ),
        array(
            'id' => 'static_header',
            'type' => 'checkbox',
            'title' => esc_html__('Static', 'trucking'),
            'subtitle' => esc_html__('Make header static.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'header_button_text',
            'type' => 'text',
            'title' => esc_html__('Button text', 'trucking'),
            'subtitle' => esc_html__('Button that appears on the right side of menu.', 'trucking'),
            'default' => esc_html__('Get a quote', 'trucking'),
            'required' => array('header_style', 'contains', 'header-style03-'),
        ),
        array(
            'id' => 'header_button_url',
            'type' => 'text',
            'title' => esc_html__('Button URL', 'trucking'),
            'subtitle' => esc_html__('Button that appears on the right side of menu.', 'trucking'),
            'default' => '',
            'required' => array('header_style', 'contains', 'header-style03-'),
        ),
        array(
            'id' => 'header_search',
            'type' => 'checkbox',
            'title' => esc_html__('Search', 'trucking'),
            'subtitle' => esc_html__('Show search button.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'header_wpml_flags',
            'type' => 'checkbox',
            'title' => esc_html__('WPML Flags', 'trucking'),
            'subtitle' => esc_html__('Show WPML language flags in header.', 'trucking'),
            'default' => '0'
        ),
        array(
            'id' => 'logo',
            'type' => 'media',
            'title' => esc_html__('Logo', 'trucking'),
            'compiler' => 'true',
            'subtitle' => esc_html__('Upload logo for your website.', 'trucking')
        ),
        array(
            'id' => 'retina_logo',
            'type' => 'media',
            'title' => esc_html__('HiDPI Logo', 'trucking'),
            'compiler' => 'true',
            'subtitle' => esc_html__('Upload HiDPI (Retina) logo for your website.', 'trucking'),
            'required' => array('retina', 'equals', '1'),
        ),
        array(
            'id' => 'top_bar_content_type',
            'type' => 'select',
            'title' => esc_html__('Top Bar content', 'trucking'),
            'subtitle' => esc_html__('Choose between Contact information or Social icons.', 'trucking'),
            'options' => array(
                'contact' => 'Contact information',
                'social' => 'Social icons',
            ), //Must provide key => value(array:title|img) pairs for radio options
            'default' => 'contact'
        ),
        array(
            'id' => 'header_social_icons',
            'type' => 'select_text',
            'title' => esc_html__('Social Icons', 'trucking'),
            'subtitle' => esc_html__('Add social icons to header.', 'trucking'),
            'icons' => volcanno_get_pixons_names(),
            'required' => array('top_bar_content_type', 'equals', 'social'),
        ),
        array(
            'id' => 'header_contact',
            'type' => 'select_text',
            'title' => esc_html__('Top Bar Contact Text', 'trucking'),
            'subtitle' => esc_html__('Icon and text will be rendered above menu.', 'trucking'),
            'icons' => volcanno_font_awesome_classes(),
            'key_prefix' => 'fa ',
            'required' => array('top_bar_content_type', 'equals', 'contact'),
        ),
        array(
            'id' => 'menu_icons',
            'type' => 'select_label',
            'title' => esc_html__('Menu Icons', 'trucking'),
            'subtitle' => esc_html__('Add icons to menu items. Can be used only on regular menu, not Mega Menu.', 'trucking'),
            'icons' => volcanno_font_awesome_classes(),
            'options' => volcanno_get_top_level_menu_items('primary'),
            'default' => '',
            'key_prefix' => 'fa '
        ),
        array(
            'id' => 'mobile_menu_text_open',
            'type' => 'text',
            'title' => esc_html__('Mobile Menu Text: "Open Menu"', 'trucking'),
            'subtitle' => esc_html__('Text that appears on mobile menu button.', 'trucking'),
            'default' => 'Open Menu',
        ),
        array(
            'id' => 'mobile_menu_text_back',
            'type' => 'text',
            'title' => esc_html__('Mobile Menu Text: "back"', 'trucking'),
            'subtitle' => esc_html__('Text that is used for back option.', 'trucking'),
            'default' => 'back',
        ),
)));

Redux::setSection($opt_name, array(
    'icon' => 'el-icon-tint',
    'title' => esc_html__('Page title', 'trucking'),
    'fields' => array(
        array(
            'id' => 'title_text_color',
            'type' => 'color',
            'title' => esc_html__('Text color', 'trucking'),
            'subtitle' => esc_html__('Set custom text color for page title.', 'trucking'),
            'default' => '#fff',
            'validate' => 'color',
            'output' => array('.page-title-style02 h1', '.page-title-style01 h1'),
            'mode' => 'color'
        ),
        array(
            'id' => '21',
            'type' => 'divide'
        ),
        array(
            'id' => 'show_breadcrumbs',
            'type' => 'switch',
            'title' => esc_html__('Breadcrumbs', 'trucking'),
            'subtitle' => esc_html__('Set breadcrumbs visibility.', 'trucking'),
            "default" => 1,
            'on' => 'Show',
            'off' => 'Hide',
        ),
        array(
            'id' => 'breadcrumbs_navigation_label',
            'type' => 'checkbox',
            'required' => array('show_breadcrumbs', '=', '1'),
            'title' => esc_html__('Breadcrumbs Text', 'trucking'),
            'subtitle' => esc_html__('Show navigation label instead page/post title.', 'trucking'),
            'default' => '0'// 1 = on | 0 = off
        ),
        array(
            'id' => 'breadcrumbs_length',
            'type' => 'text',
            'required' => array('show_breadcrumbs', '=', '1'),
            'title' => esc_html__('Breadcrumbs Length', 'trucking'),
            'desc' => esc_html__('Breadcrumbs will be cut by number of characters entered here.', 'trucking'),
            'validate' => 'numeric',
            'default' => 25
        ),
        array(
            'id' => 'breadcrumbs_prefix',
            'type' => 'text',
            'required' => array('show_breadcrumbs', '=', '1'),
            'title' => esc_html__('Breadcrumbs Prefix', 'trucking'),
            'desc' => esc_html__('Text that will be added before breadcrumbs.', 'trucking'),
            'default' => 'You are here: '
        ),
        /* VEHICLES SINGLE / CATEGORY */   
        array(
            'id' => 'section-title-portfolio-single',
            'type' => 'section',
            'title' => esc_html__('Vehicles single / category archive', 'trucking'),
            'subtitle' => esc_html__('Title on Portfolio single and category page.', 'trucking'),
            'indent' => false // Indent all options below until the next 'section' option is set.
        ),
        array(
            'id' => 'vehicles_category_show_title',
            'type' => 'switch',
            'title' => esc_html__('Show title', 'trucking'),
            'subtitle' => esc_html__('Check to show title on Portfolio category pages.', 'trucking'),
            "default" => 1,
            'on' => 'Show',
            'off' => 'Hide',
        ),
        array(
            'id' => 'vehicles_category_title_color',
            'type' => 'color',
            'title' => esc_html__('Background Color', 'trucking'),
            'subtitle' => esc_html__('Set custom color for page title background.', 'trucking'),
            'default' => '#ededed',
            'validate' => 'color',
        ),
        array(
            'id' => 'vehicles_category_title_image',
            'type' => 'media',
            'title' => esc_html__('Background Image', 'trucking'),
            'compiler' => 'true',
            'subtitle' => esc_html__('Upload image that will be use as page title background.', 'trucking')
        ),
        /* BLOG SINGLE / CATEGORY */   
        array(
            'id' => 'section-title-blog-single',
            'type' => 'section',
            'title' => esc_html__('Blog single/category/404/search...', 'trucking'),
            'subtitle' => esc_html__('Title on Blog single,  category page, 404, search results, tags etc.', 'trucking'),
            'indent' => true,            
        ),
        array(
            'id' => 'blog_category_override_blog',
            'type' => 'switch',
            'title' => esc_html__('Override settings from Blog page.', 'trucking'),
            'subtitle' => esc_html__('By default theme takes settings from Blog page and applies to single post and category pages. Enable this to override that settings.', 'trucking'),
            "default" => 0,
            'on' => 'Override',
            'off' => 'Use Blog settings',
        ),        
        array(
            'id' => 'blog_category_show_title',
            'type' => 'switch',
            'title' => esc_html__('Show title', 'trucking'),
            'subtitle' => esc_html__('Check to show title on Blog category pages.', 'trucking'),
            "default" => 1,
            'on' => 'Show',
            'off' => 'Hide',
            'required' => array('blog_category_override_blog', '=', '1')
        ),
        array(
            'id' => 'blog_category_title_color',
            'type' => 'color',
            'title' => esc_html__('Background Color', 'trucking'),
            'subtitle' => esc_html__('Set custom color for page title background.', 'trucking'),
            'default' => '#ededed',
            'validate' => 'color',
            'required' => array('blog_category_show_title', '=', '1')
        ),
        array(
            'id' => 'blog_category_title_image',
            'type' => 'media',
            'title' => esc_html__('Background Image', 'trucking'),
            'compiler' => 'true',
            'subtitle' => esc_html__('Upload image that will be use as page title background.', 'trucking'),
            'required' => array('blog_category_show_title', '=', '1')
        )
)));

Redux::setSection($opt_name, array(
    'icon' => 'el-icon-font',
    'title' => esc_html__('Typography', 'trucking'),
    'fields' => array(
        array(
            'id' => 'body_font2',
            'type' => 'typography',
            'title' => esc_html__('Body Font', 'trucking'),
            'subtitle' => esc_html__('Specify the body font properties.', 'trucking'),
            'google' => true,
            'default' => array(
                'color' => '#777',
                'font-size' => '13px',
                'font-family' => 'Open Sans',
                'font-weight' => 'Normal',
                'line-height' => '22px'
            ),
            'output' => array('body')
        ),
        array(
            'id' => 'paragraphs',
            'type' => 'typography',
            'title' => esc_html__('Paragraph Font', 'trucking'),
            'subtitle' => esc_html__('Specify the paragraph font properties.', 'trucking'),
            'google' => true,
            'default' => array(
                'color' => '#777',
                'font-size' => '13px',
                'font-family' => 'Open Sans',
                'font-weight' => 'Normal',
                'line-height' => '22px'
            ),
            'output' => array('p')
        ),
        array(
            'id' => 'heading_h1',
            'type' => 'typography',
            'title' => esc_html__('H1 Heading', 'trucking'),
            'subtitle' => esc_html__('Specify the body font properties.', 'trucking'),
            'google' => true,
            'output' => array('h1'),
            'default' => array(
                'color' => '#333333',
                'font-size' => '30px',
                'font-family' => 'Raleway',
                'font-weight' => '800',
                'line-height' => '30px'
            ),
        ),
        array(
            'id' => 'heading_h2',
            'type' => 'typography',
            'title' => esc_html__('H2 Heading', 'trucking'),
            'subtitle' => esc_html__('Specify the properties for H2.', 'trucking'),
            'google' => true,
            'output' => array('h2'),
            'default' => array(
                'color' => '#333333',
                'font-size' => '24px',
                'font-family' => 'Raleway',
                'font-weight' => '800',
                'line-height' => '26px'
            ),
        ),
        array(
            'id' => 'heading_h3',
            'type' => 'typography',
            'title' => esc_html__('H3 Heading', 'trucking'),
            'subtitle' => esc_html__('Specify the properties for H3.', 'trucking'),
            'google' => true,
            'output' => array('h3'),
            'default' => array(
                'color' => '#333333',
                'font-size' => '21px',
                'font-family' => 'Raleway',
                'font-weight' => '800',
                'line-height' => '24px'
            ),
        ),
        array(
            'id' => 'heading_h4',
            'type' => 'typography',
            'title' => esc_html__('H4 Heading', 'trucking'),
            'subtitle' => esc_html__('Specify the properties for H4.', 'trucking'),
            'google' => true,
            'output' => array('h4'),
            'default' => array(
                'color' => '#333333',
                'font-size' => '18px',
                'font-family' => 'Raleway',
                'font-weight' => '800',
                'line-height' => '22px'
            ),
        ),
        array(
            'id' => 'heading_h5',
            'type' => 'typography',
            'title' => esc_html__('H5 Heading', 'trucking'),
            'subtitle' => esc_html__('Specify the properties for H5.', 'trucking'),
            'google' => true,
            'output' => array('h5'),
            'default' => array(
                'color' => '#333333',
                'font-size' => '16px',
                'font-family' => 'Raleway',
                'font-weight' => '800',
                'line-height' => '18px'
            ),
        ),
        array(
            'id' => 'heading_h6',
            'type' => 'typography',
            'title' => esc_html__('H6 Heading', 'trucking'),
            'subtitle' => esc_html__('Specify the properties for H6.', 'trucking'),
            'google' => true,
            'output' => array('h6'),
            'default' => array(
                'color' => '#333333',
                'font-size' => '15px',
                'font-family' => 'Raleway',
                'font-weight' => '800',
                'line-height' => '18px'
            ),
        ),
)));

Redux::setSection($opt_name, array(
    'icon' => 'el-icon-book',
    'title' => esc_html__('Blog', 'trucking'),
    'fields' => array(
        array(
            'id' => 'blog_style',
            'type' => 'select_image',
            'title' => esc_html__('Blog Layout', 'trucking'),
            'subtitle' => esc_html__('Choose blog layout.', 'trucking'),
            'options' => array(
                'blog-post-standard' => array('alt' => 'Standard', 'img' => TEMPLATEURL . '/img/theme-options/blog-post-standard.JPG'),
                'blog-post-list' => array('alt' => 'List', 'img' => TEMPLATEURL . '/img/theme-options/blog-post-list.JPG'),
                'blog-post-masonry' => array('alt' => 'Masonry', 'img' => TEMPLATEURL . '/img/theme-options/blog-post-masonry.JPG')
            ), //Must provide key => value(array:title|img) pairs for radio options
            'default' => 'blog-post-standard'
        ),
        array(
            'id' => 'blog_sidebar_position',
            'type' => 'image_select',
            'title' => esc_html__('Sidebar Position', 'trucking'),
            'subtitle' => esc_html__('Sidebar on the left or on the right side.', 'trucking'),
            'options' => array(
                'left' => array('alt' => 'Left side', 'img' => ReduxFramework::$_url . 'assets/img/2cl.png'),
                'right' => array('alt' => 'Right side', 'img' => ReduxFramework::$_url . 'assets/img/2cr.png')
            ), //Must provide key => value(array:title|img) pairs for radio options
            'default' => 'right',
            'required' => array(
                array('blog_style', '=', array('blog-post-standard', 'blog-post-list'))
            )
        ),
        array(
            'id' => 'blog_post_meta',
            'type' => 'checkbox',
            'title' => esc_html__('Post meta', 'trucking'),
            'subtitle' => esc_html__('Show/Hide post tags.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'blog_image_crop',
            'type' => 'checkbox',
            'title' => esc_html__('Image Cropping', 'trucking'),
            'subtitle' => esc_html__('Set image cropping for all images in main blog page.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'blog_hide_readmore',
            'type' => 'checkbox',
            'title' => esc_html__('Read More button', 'trucking'),
            'subtitle' => esc_html__('Show Read More button.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'blog_readmore_text',
            'type' => 'text',
            'required' => array('blog_hide_readmore', '=', '1'),
            'title' => esc_html__('Read More text', 'trucking'),
            'subtitle' => esc_html__('Read More button text.', 'trucking'),
            'default' => 'Read more',
            'validate' => 'no_html'
        ),
        array(
            'id' => 'blog_animation',
            'type' => 'select',
            'title' => esc_html__('Posts Animation', 'trucking'),
            'subtitle' => esc_html__('Set blog animation.', 'trucking'),
            'options' => volcanno_animate_css_animations(),
            'default' => 'disabled',
        ),
        array(
            'id' => 'blog_sidebar_animation',
            'type' => 'select',
            'title' => esc_html__('Sidebar Animation', 'trucking'),
            'subtitle' => esc_html__('Set blog sidebar animation.', 'trucking'),
            'options' => volcanno_animate_css_animations(),
            'default' => 'disabled',
        ),
        array(
            'id' => 'section-gallery',
            'type' => 'section',
            'title' => esc_html__('Gallery post format', 'trucking'),
            'subtitle' => esc_html__('Gallery post format options', 'trucking'),
            'indent' => false // Indent all options below until the next 'section' option is set.
        ),
        array(
            'id' => 'blog_gallery_slider_autoslide',
            'type' => 'checkbox',
            'title' => esc_html__('Slider auto sliding', 'trucking'),
            'subtitle' => esc_html__('Check to enable slider auto sliding', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'blog_gallery_slider_delay',
            'type' => 'text',
            'required' => array('blog_gallery_slider_autoslide', '=', '1'),
            'title' => esc_html__('Slider delay', 'trucking'),
            'subtitle' => esc_html__('Enter time delay [ms].', 'trucking'),
            'validate' => 'numeric',
            'default' => 3000
        ),
        array(
            'id' => 'section-single',
            'type' => 'section',
            'title' => esc_html__('Single post options', 'trucking'),
            'subtitle' => esc_html__('Edit single post options.', 'trucking'),
            'indent' => false // Indent all options below until the next 'section' option is set.
        ),
        array(
            'id' => 'blog_single_meta',
            'type' => 'checkbox',
            'title' => esc_html__('Post meta', 'trucking'),
            'subtitle' => esc_html__('Show/Hide post tags.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        ),
        array(
            'id' => 'blog_single_image_crop',
            'type' => 'checkbox',
            'title' => esc_html__('Image Cropping', 'trucking'),
            'subtitle' => esc_html__('Set image cropping for all images in single post.', 'trucking'),
            'default' => '1'// 1 = on | 0 = off
        )
)));

Redux::setSection($opt_name, array(
    'icon' => 'el-icon-picture',
    'title' => esc_html__('Vehicles', 'trucking'),
    'fields' => array(
        array(
            'id' => 'vehicles_page',
            'type' => 'select',
            'title' => esc_html__('Vehicles page', 'trucking'),
            'subtitle' => esc_html__('Select vehicles page. Options for single vehicles item will be fetched from this page.', 'trucking'),
            'data' => 'pages'
        ),
        array(
            'id' => 'vehicles_category_style',
            'type' => 'select',
            'title' => esc_html__('Vehicles category page style', 'trucking'),
            'subtitle' => esc_html__('Select category page style for vehicles.', 'trucking'),
            'options' => array('full' => 'Full', 'grid' => 'Grid'),
            'default' => 'full',
        ),
)));

Redux::setSection($opt_name, array(
    'icon' => 'el-icon-adjust-alt',
    'title' => esc_html__('Footer', 'trucking'),
    'fields' => array(
        array(
            'id' => 'footer_show',
            'type' => 'switch',
            'title' => esc_html__('Show Footer', 'trucking'),
            'subtitle' => esc_html__('Show Footer section.', 'trucking'),
            "default" => 1,
            'on' => 'Show',
            'off' => 'Hide',
        ),
        array(
            'id' => 'footer_style',
            'type' => 'select',
            'title' => esc_html__('Two color versions of footer available.', 'trucking'),
            'options' => array('footer-dark' => 'Dark', 'footer-light' => 'Light'),
            'default' => 'footer-dark',
        ),
        array(
            'id' => 'footer_background',
            'type' => 'color',
            'output' => array('background-color' => '#footer-wrapper.footer-dark, #footer-wrapper.footer-light'),
            'title' => esc_html__('Footer background color', 'trucking'),
            'subtitle' => esc_html__('Set footer custom background color.', 'trucking'),
            'default' => '',
            'validate' => 'color',
        ),
        array(
            'id' => 'footer_widget_areas',
            'type' => 'slider',
            'required' => array('footer_show', '=', '1'),
            'title' => esc_html__('Widget areas', 'trucking'),
            'subtitle' => esc_html__('Number of widget areas.', 'trucking'),
            "default" => "4",
            "min" => "1",
            "step" => "1",
            "max" => "4",
        ),
        array(
            'id' => 'copyright_section',
            'type' => 'switch',
            'title' => esc_html__('Copyright section', 'trucking'),
            'subtitle' => esc_html__('Show copyright section.', 'trucking'),
            "default" => 1,
            'on' => 'Show',
            'off' => 'Hide',
        ),
        array(
            'id' => 'copyright_text',
            'type' => 'textarea',
            'title' => esc_html__('Copyright Text', 'trucking'),
            'subtitle' => esc_html__('Enter copyright text.', 'trucking'),
            'validate' => 'html', //see http://codex.wordpress.org/Function_Reference/wp_kses_post
            'default' => 'TRUCKING 2019. All RIGHTS RESERVED.',
            'required' => array('copyright_section', '=', 1),
        ),
        array(
            'id' => 'footer_bottom_right',
            'type' => 'select',
            'title' => esc_html__('Bottom Right Section', 'trucking'),
            'subtitle' => esc_html__('Content type on the bottom right section.', 'trucking'),
            'options' => array('text' => 'Regular text', 'menu' => 'Menu'),
            'default' => 'text',
            'required' => array('copyright_section', '=', 1),
        ),
        array(
            'id' => 'bottom_right_text',
            'type' => 'textarea',
            'title' => esc_html__('Bottom Right Text', 'trucking'),
            'subtitle' => esc_html__('Text that will appear on bottom right section.', 'trucking'),
            'validate' => 'html', //see http://codex.wordpress.org/Function_Reference/wp_kses_post
            'default' => 'DESIGNED AND DEVELOPED BY PIXEL INDUSTRY. ELITE AUTHOR.',
            'required' => array('footer_bottom_right', '=', 'text'),
            'required' => array('copyright_section', '=', 1),
        ),
        array(
            'id' => 'widgets_animation',
            'type' => 'select',
            'title' => esc_html__('Widgets animation', 'trucking'),
            'subtitle' => esc_html__('Set animation for widgets.', 'trucking'),
            'options' => volcanno_animate_css_animations(),
            'default' => 'disabled',
        ),
)));



/*
 * Load section for Content Maker and WooCommerce dynamically
 */
if (CONTENT_MAKER)
    add_filter("redux/options/volcanno_options/sections", 'cma_settings_fields');
