<?php

// Prevent loading file directly
if (!defined('ABSPATH')) {
    exit;
}
if (!class_exists("CMA_Shortcodes_Promo_Box")) {

    class CMA_Shortcodes_Promo_Box extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__('Promo Box', 'trucking');
            $settings['description'] = esc_html__('Promotional box where you can describe your product/service or create job ad.', 'trucking');
            $settings['shortcode'] = 'cma_promo_box';
            $settings['nested'] = false;
            $settings['selected_option'] = 'title'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function shortcode_html($atts = array(), $content = null) {
            global $vlc_is_retina, $volcanno_theme_config;

            $image_style = '';
            $color_style = '';

            extract(shortcode_atts(array(
                'style' => 'style-1',
                'title' => '',
                'text' => '',
                'button_text' => '',
                'button_url' => '',
                'color' => '',
                'image' => ''
                            ), $atts, 'cma_promo_box'));

            // sanitize title
            $title = wp_kses($title, $volcanno_theme_config['allowed_html_tags']);

            // sanitize text
            $text = wp_kses($text, $volcanno_theme_config['allowed_html_tags']);

            // image
            $image = explode(',', $image);

            //get first image
            $image_url = wp_get_attachment_image_src($image[0], 'full');

            if (!empty($color)) {
                $color_style = "style='background-color: {$color};'";
            };

            if (!empty($image_url)) {
                $image_style = "style='background-image: url($image_url[0]);'";
            };

            if ($style == 'style-1') {
                $html = "<div class='promo-box' {$image_style} {$color_style}>";
            } else {
                $html = "<div class='promo-box02' {$image_style} {$color_style}>";
            }

            if (!empty($title)) {
                if ($style == 'style-2') {
                    $html .= "<div class='custom-heading centered'><h4>{$title}</h4></div>";
                } else {
                    $html .= "<h4>{$title}</h4>";
                }
            }

            if (!empty($text)) {
                $html .= "<p>{$text}</p>";
            }

            if (!empty($button_text) && !empty($button_url)) {
                $button_url = esc_url_raw($button_url);
                $button_text = esc_html($button_text);

                if($style == 'style-2'){
                    $html .= "<a href='{$button_url}' class='btn btn-big btn-yellow btn-centered'>
                                <span>{$button_text}</span>
                            </a>";
                }else{
                    $html .= "<a href='{$button_url}' class='btn btn-medium btn-yellow'>
                                <span>{$button_text}</span>
                            </a>";
                }
                
            }

            $html .= "</div>";

            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'style' => array(
                    'type' => 'radio',
                    'title' => esc_html__('Style', 'trucking'),
                    'description' => esc_html__('Select element style.', 'trucking'),
                    'options' => array(
                        'style-1' => esc_html__('Style 1', 'trucking'),
                        'style-2' => esc_html__('Style 2', 'trucking')
                    ),
                    'default' => 'style-1'
                ),
                'title' => array(
                    'type' => 'text',
                    'title' => esc_html__('Title', 'trucking'),
                    'description' => esc_html__('Text that will appear at the top of the box.', 'trucking')
                ),
                'text' => array(
                    'type' => 'textarea',
                    'title' => esc_html__('Text', 'trucking'),
                    'description' => esc_html__('Short description below title.', 'trucking')
                ),
                'button_text' => array(
                    'type' => 'text',
                    'title' => esc_html__('Button text', 'trucking'),
                    'description' => esc_html__('Enter button text.', 'trucking')
                ),
                'button_url' => array(
                    'type' => 'text',
                    'title' => esc_html__('Button URL', 'trucking'),
                    'description' => esc_html__('Enter button URL.', 'trucking')
                ),
                'color' => array(
                    'type' => 'color_picker',
                    'title' => esc_html__('Background color', 'trucking'),
                    'description' => esc_html__('Set box background color.', 'trucking'),
                    'default' => ''
                ),
                'image' => array(
                    'type' => 'image',
                    'title' => esc_html__('Background image', 'trucking'),
                    'description' => esc_html__('Please upload background image', 'trucking')
                ),
            );
        }

    }

}
?>
