<?php

// Prevent loading file directly
if ( !defined( 'ABSPATH' ) ) {
    exit;
}
if ( !class_exists( "CMA_Shortcodes_Team_Member" ) ) {

    class CMA_Shortcodes_Team_Member extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__( 'Team Member (single)', 'trucking' );
            $settings['description'] = esc_html__( 'Your company team member (single member).', 'trucking' );
            $settings['shortcode'] = 'cma_team_member';
            $settings['nested'] = false;
            $settings['selected_option'] = 'name'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function shortcode_html( $atts = array(), $content = null ) {
            global $vlc_is_retina, $volcanno_theme_config;

            $html_members = '';
            $social_html = '';
            $member_image = '';
            $image_link = '';

            extract( shortcode_atts( array(
                'style' => '',
                'name' => '',
                'position' => '',
                'text' => '',
                'image' => '',
                'url' => ''
                            ), $atts, 'cma_team_member' ) );

            // team member image
            $image = explode( ',', $image );

            //get first image
            $image_url = wp_get_attachment_image_src( $image[0], 'full' );

            // parameters for cropping image
            $params = array( 'width' => 263, 'height' => 292 );

            // filter image size, retina check is hooked on this
            $params = apply_filters( 'cma_element_image_dimensions', $params, $this->shortcode_settings['shortcode'] );

            $name_alt = esc_attr( $name );
            
            if ( !empty( $image_url ) ) {
                $member_image = bfi_thumb( $image_url[0], $params );
                $member_image = "<img src='{$member_image}' alt='{$name_alt}'/>";
            }
            
            // text cut if it's too long
            $text = wp_kses( $text, $volcanno_theme_config['allowed_html_tags'] );

            $name = wp_kses( $name, $volcanno_theme_config['allowed_html_tags'] );

            // team member page
            if ( !empty( $url ) ) {
                $url = esc_url_raw( $url );

                $name_html = "<a href='{$url}'><h5>{$name}</h5></a>";

                if ( !empty( $member_image ) ) {
                    $image_link = "<a href='{$url}'>
                                    {$member_image}
                                </a>";
                }
            } else {
                if ( !empty( $member_image ) ) {
                    $image_link = $member_image;
                }

                $name_html = "<h5>{$name}</h5>";
            }

            $position = esc_html( $position );

            if ( $style == 'vertical' ) {
                $html = " <div class='team-member'>
                                    {$image_link}
                                    <div class='team-details'>
                                        {$name_html}
                                        <p class='position'>{$position}</p>
                                    </div>
                                </div>";
            } else {
                $html = "<div class='team-horizontal'>
                                {$image_link}

                                <div class='team-details-container'>
                                    <div class='team-details'>
                                        {$name_html}
                                        <p class='position'>{$position}</p>
                                    </div>

                                    <p>{$text}</p>
                                </div>
                            </div>";
            }


            return $html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'style' => array(
                    'type' => 'radio',
                    'title' => esc_html__( 'Style', 'trucking' ),
                    'description' => esc_html__( 'Select team member style.', 'trucking' ),
                    'options' => array(
                        'vertical' => 'Vertical',
                        'horizontal' => 'Horizontal',
                    )
                ),
                'name' => array(
                    'type' => 'text',
                    'title' => esc_html__( 'Name', 'trucking' ),
                    'description' => esc_html__( 'Team member name.', 'trucking' )
                ),
                'position' => array(
                    'type' => 'text',
                    'title' => esc_html__( 'Job position', 'trucking' ),
                    'description' => esc_html__( 'Enter job position.', 'trucking' )
                ),
                'text' => array(
                    'type' => 'textarea',
                    'title' => esc_html__( 'Text', 'trucking' ),
                    'description' => esc_html__( 'Write something about employee.', 'trucking' ),
                    'condition' => array( 'style', '==', 'horizontal' )
                ),
                'image' => array(
                    'type' => 'image',
                    'title' => esc_html__( 'Image', 'trucking' ),
                    'description' => esc_html__( 'Please upload team member image', 'trucking' )
                ),
                'url' => array(
                    'type' => 'text',
                    'title' => esc_html__( 'Page URL', 'trucking' ),
                    'description' => esc_html__( 'Link to Team member personal page (leave empty to disable link).', 'trucking' )
                )
            );
        }

    }

}
?>
