<?php

// Prevent loading file directly
if ( !defined( 'ABSPATH' ) ) {
    exit;
}
if ( !class_exists( "CMA_Shortcodes_Testimonials_Carousel" ) ) {

    class CMA_Shortcodes_Testimonials_Carousel extends Content_Maker_Shortcodes {

        function shortcode_block_info() {
            $settings['title'] = esc_html__( 'Testimonials Carousel', 'trucking' );
            $settings['description'] = esc_html__( 'Simple testimonials carousel with picture.', 'trucking' );
            $settings['shortcode'] = 'cma_testimonials_carousel';
            $settings['nested'] = false;
            $settings['selected_option'] = 'testimonials|author'; // enter field ID - show this field's value on element on grid - easier to find desired element

            $this->shortcode_settings = $settings;
        }

        function enqueue_scripts() {
            /* scripts */
            wp_enqueue_script( 'cma-scripts-testimonials-carousel', CMA_CONTENT_URL . 'testimonials_carousel/js/testimonials_carousel.js', array( 'jquery', 'owl-carousel' ), '1.0', TRUE );
        }

        function shortcode_html( $atts = array(), $content = null ) {
            global $vlc_is_retina;

            $i = 1;
            $testimonials_html = '';
            $nav_class = '';

            extract( shortcode_atts( array(
                'testimonials' => '',
                'auto_scroll' => '0',
                'timeout' => '3000',
                'nav' => 'dots'
                            ), $atts, 'cma_testimonials_carousel' ) );

            if ( $nav == 'nav' )
                $nav_class = 'owl-carousel-navigation';

            $testimonials_html .= "<div class='carousel-container'><div class='testimonials-carousel owl-carousel {$nav_class}' data-play='{$auto_scroll}' data-timeout='{$timeout}' data-navigation='{$nav}'>";

            foreach ( $testimonials as $index => $testimonial ) {
                $author_image = '';
                $date = '';

                // execute any shortcode that might be in text
                $content = do_shortcode( $testimonial['text'] );

                // get image if available
                $images = isset( $testimonial['image'] ) ? $testimonial['image'] : '';
                if ( !empty( $images ) ) {
                    // array of images
                    $image = explode( ',', $images );

                    //get first image
                    $image_url = wp_get_attachment_image_src( $image[0], 'full' );

                    // crop image
                    $params = array( 'width' => 97, 'height' => 97 );

                    // filter image size, retina check is hooked on this
                    $params = apply_filters( 'cma_element_image_dimensions', $params, $this->shortcode_settings['shortcode'] );

                    $author_image = bfi_thumb( $image_url[0], $params );
                    $alt = esc_attr( $testimonial['author'] );
                    $author_image = "<img src='{$author_image}' alt='{$alt}'>";
                }

                $author = esc_html( $testimonial['author'] );
                $position = esc_html( $testimonial['position'] );

                // carousel testimonial
                $testimonials_html .= "<div class='owl-item'>
                                    <div class='testimonial '>
                                        <p>{$content}</p>
                                        <div class='testimonial-author'><p>{$author}, <br/>{$position}</p></div>
                                    </div>
                                </div>";
            }

            $testimonials_html .= "</div></div>";

            return $testimonials_html;
        }

        function shortcode_options_fields() {

            $this->shortcode_options = array(
                'auto_scroll' => array(
                    'type' => 'checkbox',
                    'title' => esc_html__( 'Auto Scroll', 'trucking' ),
                    'description' => esc_html__( 'Set testimonial to auto scroll.', 'trucking' ),
                    'default' => '0'
                ),
                'timeout' => array(
                    'type' => 'number',
                    'title' => esc_html__( 'Scroll Timeout', 'trucking' ),
                    'description' => esc_html__( 'Enter scrolling timeout.', 'trucking' ),
                    'default' => '3000',
                    'options' => array(
                        'step' => 10
                    )
                ),
                'nav' => array(
                    'type' => 'select',
                    'title' => esc_html__( 'Navigation', 'trucking' ),
                    'description' => esc_html__( 'Select navigation type', 'trucking' ),
                    'default' => '3000',
                    'options' => array(
                        'none' => 'None',
                        'nav' => 'Prev & Next',
                        'dots' => 'Dots'
                    )
                ),
                'testimonials' => array(
                    'type' => 'group',
                    'title' => esc_html__( 'Testimonials', 'trucking' ),
                    'max' => '',
                    'options' => array(
                        'author' => array(
                            'type' => 'text',
                            'title' => esc_html__( 'Author', 'trucking' ),
                            'description' => esc_html__( 'Testimonial author name.', 'trucking' )
                        ),
                        'position' => array(
                            'type' => 'text',
                            'title' => esc_html__( 'Position', 'trucking' ),
                            'description' => esc_html__( 'Testimonial author position.', 'trucking' )
                        ),
                        'text' => array(
                            'type' => 'textarea',
                            'title' => esc_html__( 'Text', 'trucking' ),
                            'description' => esc_html__( 'Testimonial text.', 'trucking' )
                        ),
                        'date' => array(
                            'type' => 'text',
                            'title' => esc_html__( 'Date', 'trucking' ),
                            'description' => esc_html__( 'Testimonial date.', 'trucking' ),
                            'condition' => array( 'type', '!=', 'testimonial-simple' )
                        ),
                    )
                )
            );
        }

    }

}
?>
