<?php

/**
 *
 * @package	   TGM-Plugin-Activation
 * @subpackage Example
 * @version	   2.3.6
 * @author	   Thomas Griffin <thomas@thomasgriffinmedia.com>
 * @author	   Gary Jones <gamajo@gamajo.com>
 * @copyright  Copyright (c) 2012, Thomas Griffin
 * @license	   http://opensource.org/licenses/gpl-2.0.php GPL v2 or later
 * @link       https://github.com/thomasgriffin/TGM-Plugin-Activation
 */
/**
 * Include the TGM_Plugin_Activation class.
 */

define('THEMENAME', 'volcanno');

require_once get_template_directory() . '/core/libs/external/class-tgm-plugin-activation.php';

add_action('tgmpa_register', 'volcanno_register_required_plugins');

/**
 * Register the required plugins for this theme.
 *
 * In this example, we register two plugins - one included with the TGMPA library
 * and one from the .org repo.
 *
 * The variable passed to tgmpa_register_plugins() should be an array of plugin
 * arrays.
 *
 * This function is hooked into tgmpa_init, which is fired within the
 * TGM_Plugin_Activation class constructor.
 */
function volcanno_register_required_plugins() {

    /**
     * Array of plugin arrays. Required keys are name and slug.
     * If the source is NOT from the .org repo, then source is also required.
     */
    $plugins = array(
        // Required and optional plugins
        array(
            'name' => 'WordPress Importer',
            'slug' => 'wordpress-importer',
            'required' => true,
            'version' => '0.6.1'
        ),
        array(
            'name' => 'Disable Gutenberg',
            'slug' => 'disable-gutenberg',
            'required' => true,
            'version' => ''
        ),
        array(
            'name' => 'Custom Post Types',
            'slug' => 'volcanno-custom-post-types',
            'required' => true,
            'version' => '1.0',
            'source' => 'volcanno-custom-post-types.zip'
        ),
        array(
            'name' => 'Meta Box',
            'slug' => 'meta-box',
            'required' => true,
            'version' => ''
        ),
        array(
            'name' => 'Meta Box Tabs',
            'slug' => 'meta-box-tabs',
            'required' => true,
            'version' => '0.1.5',
            'source' => 'meta-box-tabs.zip'
        ),
        array(
            'name' => 'Meta Box Conditional Logic',
            'slug' => 'meta-box-conditional-logic',
            'required' => true,
            'version' => '1.3.1',
            'source' => 'meta-box-conditional-logic.zip'
        ),
        array(
            'name' => 'Contact Form 7',
            'slug' => 'contact-form-7',
            'required' => false,
            'version' => ''
        ),
        array(
            'name' => 'Newsletter (Optional)',
            'slug' => 'newsletter',
            'required' => false,
            'version' => ''
        ),
        array(
            'name' => 'MCE Table Buttons (Optional)',
            'slug' => 'mce-table-buttons',
            'required' => false,
            'version' => ''
        ),
        array(
            'name' => 'Black Studio TinyMCE Widget (Optional)',
            'slug' => 'black-studio-tinymce-widget',
            'required' => false,
            'version' => ''
        ),
        array(
            'name' => 'Flexi Pages Widget (Optional)',
            'slug' => 'flexi-pages-widget',
            'required' => false,
            'version' => ''
        ),
        array(
            'name' => 'Content Maker', // The plugin name
            'slug' => 'content-maker', // The plugin slug (typically the folder name)
            'source' => 'content-maker.zip', // The plugin source
            'required' => true, // If false, the plugin is only 'recommended' instead of required
            'version' => '1.3.5', // E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented
            'force_activation' => false, // If true, plugin is activated upon theme activation and cannot be deactivated until theme switch
            'force_deactivation' => false, // If true, plugin is deactivated upon theme switch, useful for theme-specific plugins
            'external_url' => '', // If set, overrides default API URL and points to an external URL
        ),
        array(
            'name' => 'Master Slider', // The plugin name
            'slug' => 'masterslider', // The plugin slug (typically the folder name)
            'source' => 'masterslider.zip', // The plugin source
            'required' => false, // If false, the plugin is only 'recommended' instead of required
            'version' => '3.2.7', // E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented
            'force_activation' => false, // If true, plugin is activated upon theme activation and cannot be deactivated until theme switch
            'force_deactivation' => false, // If true, plugin is deactivated upon theme switch, useful for theme-specific plugins
            'external_url' => '', // If set, overrides default API URL and points to an external URL
        ),
        array(
            'name' => 'TweetScroll Widget (Optional)', // The plugin name
            'slug' => 'tweetscroll-widget', // The plugin slug (typically the folder name)            
            'required' => false, // If false, the plugin is only 'recommended' instead of required
            'version' => '' // E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented            
        ),
        array(
            'name' => 'Simple Share Buttons Adder (Optional)', // The plugin name
            'slug' => 'simple-share-buttons-adder', // The plugin slug (typically the folder name)            
            'required' => false, // If false, the plugin is only 'recommended' instead of required
            'version' => '' // E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented            
        ),
        array(
            'name' => 'Envato Market - Updates (Optional)', // The plugin name
            'slug' => 'envato-market', // The plugin slug (typically the folder name)   
            'source' => 'envato-market.zip', // The plugin source
            'required' => false, // If false, the plugin is only 'recommended' instead of required
            'version' => '1.0.0-RC2' // E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented            
        ),
        array(
            'name' => 'SVG Support', // The plugin name
            'slug' => 'svg-support', // The plugin slug (typically the folder name)            
            'required' => true, // If false, the plugin is only 'recommended' instead of required
            'version' => '' // E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented            
        )
    );
    
    // Change this to your theme text domain, used for internationalising strings
    $theme_text_domain = 'trucking';

    /**
     * Array of configuration settings. Amend each line as needed.
     * If you want the default strings to be available under your own theme domain,
     * leave the strings uncommented.
     * Some of the strings are added into a sprintf, so see the comments at the
     * end of each line for what each argument will be.
     */
    
    $config = array(
        'domain' => $theme_text_domain, // Text domain - likely want to be the same as your theme.
        'default_path' => get_template_directory() . '/includes/plugins/', // Default absolute path to pre-packaged plugins
        'parent_slug' => 'volcanno_options', // Default parent menu slug
        'menu' => 'install-required-plugins', // Menu slug
        'has_notices' => true, // Show admin notices or not
        'is_automatic' => true, // Automatically activate plugins after installation or not
        'message' => '', // Message to output right before the plugins table
        'strings' => array(
            'page_title' => esc_html__('Install Required Plugins', 'trucking'),
            'menu_title' => esc_html__('Install Plugins', 'trucking'),
            'installing' => esc_html__('Installing Plugin: %s', 'trucking'), // %1$s = plugin name
            'oops' => esc_html__('Something went wrong with the plugin API.', 'trucking'),
            'notice_can_install_required' => _n_noop('This theme requires the following plugin: %1$s.', 'This theme requires the following plugins: %1$s.', 'trucking'), // %1$s = plugin name(s)
            'notice_can_install_recommended' => _n_noop('This theme recommends the following plugin: %1$s.', 'This theme recommends the following plugins: %1$s.', 'trucking'), // %1$s = plugin name(s)
            'notice_cannot_install' => _n_noop('Sorry, but you do not have the correct permissions to install the %s plugin. Contact the administrator of this site for help on getting the plugin installed.', 'Sorry, but you do not have the correct permissions to install the %s plugins. Contact the administrator of this site for help on getting the plugins installed.', 'trucking'), // %1$s = plugin name(s)
            'notice_can_activate_required' => _n_noop('The following required plugin is currently inactive: %1$s.', 'The following required plugins are currently inactive: %1$s.', 'trucking'), // %1$s = plugin name(s)
            'notice_can_activate_recommended' => _n_noop('The following recommended plugin is currently inactive: %1$s.', 'The following recommended plugins are currently inactive: %1$s.', 'trucking'), // %1$s = plugin name(s)
            'notice_cannot_activate' => _n_noop('Sorry, but you do not have the correct permissions to activate the %s plugin. Contact the administrator of this site for help on getting the plugin activated.', 'Sorry, but you do not have the correct permissions to activate the %s plugins. Contact the administrator of this site for help on getting the plugins activated.', 'trucking'), // %1$s = plugin name(s)
            'notice_ask_to_update' => _n_noop('The following plugin needs to be updated to its latest version to ensure maximum compatibility with this theme: %1$s.', 'The following plugins need to be updated to their latest version to ensure maximum compatibility with this theme: %1$s.', 'trucking'), // %1$s = plugin name(s)
            'notice_cannot_update' => _n_noop('Sorry, but you do not have the correct permissions to update the %s plugin. Contact the administrator of this site for help on getting the plugin updated.', 'Sorry, but you do not have the correct permissions to update the %s plugins. Contact the administrator of this site for help on getting the plugins updated.', 'trucking'), // %1$s = plugin name(s)
            'install_link' => _n_noop('Begin installing plugin', 'Begin installing plugins', 'trucking'),
            'activate_link' => _n_noop('Activate installed plugin', 'Activate installed plugins', 'trucking'),
            'return' => esc_html__('Return to Required Plugins Installer', 'trucking'),
            'plugin_activated' => esc_html__('Plugin activated successfully.', 'trucking'),
            'complete' => esc_html__('All plugins installed and activated successfully. %s', 'trucking'), // %1$s = dashboard link
            'nag_type' => 'updated' // Determines admin notice type - can only be 'updated' or 'error'
        )
    );

    tgmpa($plugins, $config);
}
